# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
from typing import Any, Dict, List, Optional, Union

import msrest.serialization

from ._app_platform_management_client_enums import *


class ActiveDeploymentCollection(msrest.serialization.Model):
    """Object that includes an array of Deployment resource name and set them as active.

    :ivar active_deployment_names: Collection of Deployment name.
    :vartype active_deployment_names: list[str]
    """

    _attribute_map = {
        'active_deployment_names': {'key': 'activeDeploymentNames', 'type': '[str]'},
    }

    def __init__(
        self,
        *,
        active_deployment_names: Optional[List[str]] = None,
        **kwargs
    ):
        """
        :keyword active_deployment_names: Collection of Deployment name.
        :paramtype active_deployment_names: list[str]
        """
        super(ActiveDeploymentCollection, self).__init__(**kwargs)
        self.active_deployment_names = active_deployment_names


class ApplicationInsightsAgentVersions(msrest.serialization.Model):
    """Application Insights agent versions properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar java: Indicates the version of application insight java agent.
    :vartype java: str
    """

    _validation = {
        'java': {'readonly': True},
    }

    _attribute_map = {
        'java': {'key': 'java', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(ApplicationInsightsAgentVersions, self).__init__(**kwargs)
        self.java = None


class Resource(msrest.serialization.Model):
    """The core properties of ARM resources.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(Resource, self).__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None
        self.system_data = None


class ProxyResource(Resource):
    """The resource model definition for a ARM proxy resource. It will have everything other than required location and tags.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(ProxyResource, self).__init__(**kwargs)


class AppResource(ProxyResource):
    """App resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the App resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.AppResourceProperties
    :ivar identity: The Managed Identity type of the app resource.
    :vartype identity: ~azure.mgmt.appplatform.v2022_04_01.models.ManagedIdentityProperties
    :ivar location: The GEO location of the application, always the same with its parent resource.
    :vartype location: str
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'AppResourceProperties'},
        'identity': {'key': 'identity', 'type': 'ManagedIdentityProperties'},
        'location': {'key': 'location', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        properties: Optional["AppResourceProperties"] = None,
        identity: Optional["ManagedIdentityProperties"] = None,
        location: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the App resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.AppResourceProperties
        :keyword identity: The Managed Identity type of the app resource.
        :paramtype identity: ~azure.mgmt.appplatform.v2022_04_01.models.ManagedIdentityProperties
        :keyword location: The GEO location of the application, always the same with its parent
         resource.
        :paramtype location: str
        """
        super(AppResource, self).__init__(**kwargs)
        self.properties = properties
        self.identity = identity
        self.location = location


class AppResourceCollection(msrest.serialization.Model):
    """Object that includes an array of App resources and a possible link for next set.

    :ivar value: Collection of App resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.AppResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[AppResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["AppResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of App resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.AppResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(AppResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class AppResourceProperties(msrest.serialization.Model):
    """App resource properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar public: Indicates whether the App exposes public endpoint.
    :vartype public: bool
    :ivar url: URL of the App.
    :vartype url: str
    :ivar addon_configs: Collection of addons.
    :vartype addon_configs: dict[str, dict[str, any]]
    :ivar provisioning_state: Provisioning state of the App. Possible values include: "Succeeded",
     "Failed", "Creating", "Updating", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.AppResourceProvisioningState
    :ivar fqdn: Fully qualified dns Name.
    :vartype fqdn: str
    :ivar https_only: Indicate if only https is allowed.
    :vartype https_only: bool
    :ivar temporary_disk: Temporary disk settings.
    :vartype temporary_disk: ~azure.mgmt.appplatform.v2022_04_01.models.TemporaryDisk
    :ivar persistent_disk: Persistent disk settings.
    :vartype persistent_disk: ~azure.mgmt.appplatform.v2022_04_01.models.PersistentDisk
    :ivar enable_end_to_end_tls: Indicate if end to end TLS is enabled.
    :vartype enable_end_to_end_tls: bool
    :ivar loaded_certificates: Collection of loaded certificates.
    :vartype loaded_certificates:
     list[~azure.mgmt.appplatform.v2022_04_01.models.LoadedCertificate]
    """

    _validation = {
        'url': {'readonly': True},
        'provisioning_state': {'readonly': True},
    }

    _attribute_map = {
        'public': {'key': 'public', 'type': 'bool'},
        'url': {'key': 'url', 'type': 'str'},
        'addon_configs': {'key': 'addonConfigs', 'type': '{{object}}'},
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'fqdn': {'key': 'fqdn', 'type': 'str'},
        'https_only': {'key': 'httpsOnly', 'type': 'bool'},
        'temporary_disk': {'key': 'temporaryDisk', 'type': 'TemporaryDisk'},
        'persistent_disk': {'key': 'persistentDisk', 'type': 'PersistentDisk'},
        'enable_end_to_end_tls': {'key': 'enableEndToEndTLS', 'type': 'bool'},
        'loaded_certificates': {'key': 'loadedCertificates', 'type': '[LoadedCertificate]'},
    }

    def __init__(
        self,
        *,
        public: Optional[bool] = None,
        addon_configs: Optional[Dict[str, Dict[str, Any]]] = None,
        fqdn: Optional[str] = None,
        https_only: Optional[bool] = False,
        temporary_disk: Optional["TemporaryDisk"] = None,
        persistent_disk: Optional["PersistentDisk"] = None,
        enable_end_to_end_tls: Optional[bool] = False,
        loaded_certificates: Optional[List["LoadedCertificate"]] = None,
        **kwargs
    ):
        """
        :keyword public: Indicates whether the App exposes public endpoint.
        :paramtype public: bool
        :keyword addon_configs: Collection of addons.
        :paramtype addon_configs: dict[str, dict[str, any]]
        :keyword fqdn: Fully qualified dns Name.
        :paramtype fqdn: str
        :keyword https_only: Indicate if only https is allowed.
        :paramtype https_only: bool
        :keyword temporary_disk: Temporary disk settings.
        :paramtype temporary_disk: ~azure.mgmt.appplatform.v2022_04_01.models.TemporaryDisk
        :keyword persistent_disk: Persistent disk settings.
        :paramtype persistent_disk: ~azure.mgmt.appplatform.v2022_04_01.models.PersistentDisk
        :keyword enable_end_to_end_tls: Indicate if end to end TLS is enabled.
        :paramtype enable_end_to_end_tls: bool
        :keyword loaded_certificates: Collection of loaded certificates.
        :paramtype loaded_certificates:
         list[~azure.mgmt.appplatform.v2022_04_01.models.LoadedCertificate]
        """
        super(AppResourceProperties, self).__init__(**kwargs)
        self.public = public
        self.url = None
        self.addon_configs = addon_configs
        self.provisioning_state = None
        self.fqdn = fqdn
        self.https_only = https_only
        self.temporary_disk = temporary_disk
        self.persistent_disk = persistent_disk
        self.enable_end_to_end_tls = enable_end_to_end_tls
        self.loaded_certificates = loaded_certificates


class AvailableOperations(msrest.serialization.Model):
    """Available operations of the service.

    :ivar value: Collection of available operation details.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.OperationDetail]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[OperationDetail]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["OperationDetail"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of available operation details.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.OperationDetail]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(AvailableOperations, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class AvailableRuntimeVersions(msrest.serialization.Model):
    """AvailableRuntimeVersions.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar value: A list of all supported runtime versions.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.SupportedRuntimeVersion]
    """

    _validation = {
        'value': {'readonly': True},
    }

    _attribute_map = {
        'value': {'key': 'value', 'type': '[SupportedRuntimeVersion]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(AvailableRuntimeVersions, self).__init__(**kwargs)
        self.value = None


class BindingResource(ProxyResource):
    """Binding resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the Binding resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BindingResourceProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'BindingResourceProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["BindingResourceProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the Binding resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BindingResourceProperties
        """
        super(BindingResource, self).__init__(**kwargs)
        self.properties = properties


class BindingResourceCollection(msrest.serialization.Model):
    """Object that includes an array of Binding resources and a possible link for next set.

    :ivar value: Collection of Binding resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BindingResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[BindingResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["BindingResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Binding resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BindingResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(BindingResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class BindingResourceProperties(msrest.serialization.Model):
    """Binding resource properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar resource_name: The name of the bound resource.
    :vartype resource_name: str
    :ivar resource_type: The standard Azure resource type of the bound resource.
    :vartype resource_type: str
    :ivar resource_id: The Azure resource id of the bound resource.
    :vartype resource_id: str
    :ivar key: The key of the bound resource.
    :vartype key: str
    :ivar binding_parameters: Binding parameters of the Binding resource.
    :vartype binding_parameters: dict[str, any]
    :ivar generated_properties: The generated Spring Boot property file for this binding. The
     secret will be deducted.
    :vartype generated_properties: str
    :ivar created_at: Creation time of the Binding resource.
    :vartype created_at: str
    :ivar updated_at: Update time of the Binding resource.
    :vartype updated_at: str
    """

    _validation = {
        'resource_name': {'readonly': True},
        'resource_type': {'readonly': True},
        'generated_properties': {'readonly': True},
        'created_at': {'readonly': True},
        'updated_at': {'readonly': True},
    }

    _attribute_map = {
        'resource_name': {'key': 'resourceName', 'type': 'str'},
        'resource_type': {'key': 'resourceType', 'type': 'str'},
        'resource_id': {'key': 'resourceId', 'type': 'str'},
        'key': {'key': 'key', 'type': 'str'},
        'binding_parameters': {'key': 'bindingParameters', 'type': '{object}'},
        'generated_properties': {'key': 'generatedProperties', 'type': 'str'},
        'created_at': {'key': 'createdAt', 'type': 'str'},
        'updated_at': {'key': 'updatedAt', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        resource_id: Optional[str] = None,
        key: Optional[str] = None,
        binding_parameters: Optional[Dict[str, Any]] = None,
        **kwargs
    ):
        """
        :keyword resource_id: The Azure resource id of the bound resource.
        :paramtype resource_id: str
        :keyword key: The key of the bound resource.
        :paramtype key: str
        :keyword binding_parameters: Binding parameters of the Binding resource.
        :paramtype binding_parameters: dict[str, any]
        """
        super(BindingResourceProperties, self).__init__(**kwargs)
        self.resource_name = None
        self.resource_type = None
        self.resource_id = resource_id
        self.key = key
        self.binding_parameters = binding_parameters
        self.generated_properties = None
        self.created_at = None
        self.updated_at = None


class Build(ProxyResource):
    """Build resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the build resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'BuildProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["BuildProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the build resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildProperties
        """
        super(Build, self).__init__(**kwargs)
        self.properties = properties


class BuildCollection(msrest.serialization.Model):
    """Object that includes an array of Build resources and a possible link for next set.

    :ivar value: Collection of Build resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.Build]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[Build]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["Build"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Build resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.Build]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(BuildCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class BuilderProperties(msrest.serialization.Model):
    """KPack Builder properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar provisioning_state: Builder provision status. Possible values include: "Creating",
     "Updating", "Succeeded", "Failed", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.BuilderProvisioningState
    :ivar stack: Builder cluster stack property.
    :vartype stack: ~azure.mgmt.appplatform.v2022_04_01.models.StackProperties
    :ivar buildpack_groups: Builder buildpack groups.
    :vartype buildpack_groups:
     list[~azure.mgmt.appplatform.v2022_04_01.models.BuildpacksGroupProperties]
    """

    _validation = {
        'provisioning_state': {'readonly': True},
    }

    _attribute_map = {
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'stack': {'key': 'stack', 'type': 'StackProperties'},
        'buildpack_groups': {'key': 'buildpackGroups', 'type': '[BuildpacksGroupProperties]'},
    }

    def __init__(
        self,
        *,
        stack: Optional["StackProperties"] = None,
        buildpack_groups: Optional[List["BuildpacksGroupProperties"]] = None,
        **kwargs
    ):
        """
        :keyword stack: Builder cluster stack property.
        :paramtype stack: ~azure.mgmt.appplatform.v2022_04_01.models.StackProperties
        :keyword buildpack_groups: Builder buildpack groups.
        :paramtype buildpack_groups:
         list[~azure.mgmt.appplatform.v2022_04_01.models.BuildpacksGroupProperties]
        """
        super(BuilderProperties, self).__init__(**kwargs)
        self.provisioning_state = None
        self.stack = stack
        self.buildpack_groups = buildpack_groups


class BuilderResource(ProxyResource):
    """KPack Builder resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Property of the Builder resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuilderProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'BuilderProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["BuilderProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Property of the Builder resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuilderProperties
        """
        super(BuilderResource, self).__init__(**kwargs)
        self.properties = properties


class BuilderResourceCollection(msrest.serialization.Model):
    """Object that includes an array of Builder resources and a possible link for next set.

    :ivar value: Collection of Builder resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuilderResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[BuilderResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["BuilderResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Builder resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuilderResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(BuilderResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class BuildpackBindingLaunchProperties(msrest.serialization.Model):
    """Buildpack Binding Launch Properties.

    :ivar properties: Non-sensitive properties for launchProperties.
    :vartype properties: dict[str, str]
    :ivar secrets: Sensitive properties for launchProperties.
    :vartype secrets: dict[str, str]
    """

    _attribute_map = {
        'properties': {'key': 'properties', 'type': '{str}'},
        'secrets': {'key': 'secrets', 'type': '{str}'},
    }

    def __init__(
        self,
        *,
        properties: Optional[Dict[str, str]] = None,
        secrets: Optional[Dict[str, str]] = None,
        **kwargs
    ):
        """
        :keyword properties: Non-sensitive properties for launchProperties.
        :paramtype properties: dict[str, str]
        :keyword secrets: Sensitive properties for launchProperties.
        :paramtype secrets: dict[str, str]
        """
        super(BuildpackBindingLaunchProperties, self).__init__(**kwargs)
        self.properties = properties
        self.secrets = secrets


class BuildpackBindingProperties(msrest.serialization.Model):
    """Properties of a buildpack binding.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar binding_type: Buildpack Binding Type. Possible values include: "ApplicationInsights",
     "ApacheSkyWalking", "AppDynamics", "Dynatrace", "NewRelic", "ElasticAPM".
    :vartype binding_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.BindingType
    :ivar provisioning_state: State of the Buildpack Binding. Possible values include: "Creating",
     "Updating", "Succeeded", "Failed", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.BuildpackBindingProvisioningState
    :ivar launch_properties: The object describes the buildpack binding launch properties.
    :vartype launch_properties:
     ~azure.mgmt.appplatform.v2022_04_01.models.BuildpackBindingLaunchProperties
    """

    _validation = {
        'provisioning_state': {'readonly': True},
    }

    _attribute_map = {
        'binding_type': {'key': 'bindingType', 'type': 'str'},
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'launch_properties': {'key': 'launchProperties', 'type': 'BuildpackBindingLaunchProperties'},
    }

    def __init__(
        self,
        *,
        binding_type: Optional[Union[str, "BindingType"]] = None,
        launch_properties: Optional["BuildpackBindingLaunchProperties"] = None,
        **kwargs
    ):
        """
        :keyword binding_type: Buildpack Binding Type. Possible values include: "ApplicationInsights",
         "ApacheSkyWalking", "AppDynamics", "Dynatrace", "NewRelic", "ElasticAPM".
        :paramtype binding_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.BindingType
        :keyword launch_properties: The object describes the buildpack binding launch properties.
        :paramtype launch_properties:
         ~azure.mgmt.appplatform.v2022_04_01.models.BuildpackBindingLaunchProperties
        """
        super(BuildpackBindingProperties, self).__init__(**kwargs)
        self.binding_type = binding_type
        self.provisioning_state = None
        self.launch_properties = launch_properties


class BuildpackBindingResource(ProxyResource):
    """Buildpack Binding Resource object.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of a buildpack binding.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildpackBindingProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'BuildpackBindingProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["BuildpackBindingProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of a buildpack binding.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildpackBindingProperties
        """
        super(BuildpackBindingResource, self).__init__(**kwargs)
        self.properties = properties


class BuildpackBindingResourceCollection(msrest.serialization.Model):
    """Object that includes an array of BuildpackBinding resources and a possible link for next set.

    :ivar value: Collection of BuildpackBinding resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildpackBindingResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[BuildpackBindingResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["BuildpackBindingResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of BuildpackBinding resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildpackBindingResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(BuildpackBindingResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class BuildpackProperties(msrest.serialization.Model):
    """Buildpack properties payload.

    :ivar id: Id of the buildpack.
    :vartype id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword id: Id of the buildpack.
        :paramtype id: str
        """
        super(BuildpackProperties, self).__init__(**kwargs)
        self.id = id


class BuildpacksGroupProperties(msrest.serialization.Model):
    """Buildpack group properties of the Builder.

    :ivar name: Buildpack group name.
    :vartype name: str
    :ivar buildpacks: Buildpacks in the buildpack group.
    :vartype buildpacks: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildpackProperties]
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'buildpacks': {'key': 'buildpacks', 'type': '[BuildpackProperties]'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        buildpacks: Optional[List["BuildpackProperties"]] = None,
        **kwargs
    ):
        """
        :keyword name: Buildpack group name.
        :paramtype name: str
        :keyword buildpacks: Buildpacks in the buildpack group.
        :paramtype buildpacks: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildpackProperties]
        """
        super(BuildpacksGroupProperties, self).__init__(**kwargs)
        self.name = name
        self.buildpacks = buildpacks


class BuildProperties(msrest.serialization.Model):
    """Build resource properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar relative_path: The relative path of source code.
    :vartype relative_path: str
    :ivar builder: The resource id of builder to build the source code.
    :vartype builder: str
    :ivar agent_pool: The resource id of agent pool.
    :vartype agent_pool: str
    :ivar provisioning_state: Provisioning state of the KPack build result. Possible values
     include: "Creating", "Updating", "Succeeded", "Failed", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.BuildProvisioningState
    :ivar env: The environment variables for this build.
    :vartype env: dict[str, str]
    :ivar triggered_build_result: The build result triggered by this build.
    :vartype triggered_build_result:
     ~azure.mgmt.appplatform.v2022_04_01.models.TriggeredBuildResult
    """

    _validation = {
        'provisioning_state': {'readonly': True},
        'triggered_build_result': {'readonly': True},
    }

    _attribute_map = {
        'relative_path': {'key': 'relativePath', 'type': 'str'},
        'builder': {'key': 'builder', 'type': 'str'},
        'agent_pool': {'key': 'agentPool', 'type': 'str'},
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'env': {'key': 'env', 'type': '{str}'},
        'triggered_build_result': {'key': 'triggeredBuildResult', 'type': 'TriggeredBuildResult'},
    }

    def __init__(
        self,
        *,
        relative_path: Optional[str] = None,
        builder: Optional[str] = None,
        agent_pool: Optional[str] = None,
        env: Optional[Dict[str, str]] = None,
        **kwargs
    ):
        """
        :keyword relative_path: The relative path of source code.
        :paramtype relative_path: str
        :keyword builder: The resource id of builder to build the source code.
        :paramtype builder: str
        :keyword agent_pool: The resource id of agent pool.
        :paramtype agent_pool: str
        :keyword env: The environment variables for this build.
        :paramtype env: dict[str, str]
        """
        super(BuildProperties, self).__init__(**kwargs)
        self.relative_path = relative_path
        self.builder = builder
        self.agent_pool = agent_pool
        self.provisioning_state = None
        self.env = env
        self.triggered_build_result = None


class BuildResult(ProxyResource):
    """Build result resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the build result resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildResultProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'BuildResultProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["BuildResultProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the build result resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildResultProperties
        """
        super(BuildResult, self).__init__(**kwargs)
        self.properties = properties


class BuildResultCollection(msrest.serialization.Model):
    """Object that includes an array of Build result resources and a possible link for next set.

    :ivar value: Collection of Build result resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildResult]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[BuildResult]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["BuildResult"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Build result resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildResult]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(BuildResultCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class BuildResultLog(msrest.serialization.Model):
    """Build result log resource properties payload.

    :ivar blob_url: The public download URL of this build result log.
    :vartype blob_url: str
    """

    _attribute_map = {
        'blob_url': {'key': 'blobUrl', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        blob_url: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword blob_url: The public download URL of this build result log.
        :paramtype blob_url: str
        """
        super(BuildResultLog, self).__init__(**kwargs)
        self.blob_url = blob_url


class BuildResultProperties(msrest.serialization.Model):
    """Build result resource properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: The name of this build result.
    :vartype name: str
    :ivar provisioning_state: Provisioning state of the KPack build result. Possible values
     include: "Queuing", "Building", "Succeeded", "Failed", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.BuildResultProvisioningState
    :ivar build_pod_name: The build pod name which can be used to get the build log streaming.
    :vartype build_pod_name: str
    :ivar build_stages: All of the build stage (init-container and container) resources in build
     pod.
    :vartype build_stages: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildStageProperties]
    """

    _validation = {
        'provisioning_state': {'readonly': True},
        'build_stages': {'readonly': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'build_pod_name': {'key': 'buildPodName', 'type': 'str'},
        'build_stages': {'key': 'buildStages', 'type': '[BuildStageProperties]'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        build_pod_name: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword name: The name of this build result.
        :paramtype name: str
        :keyword build_pod_name: The build pod name which can be used to get the build log streaming.
        :paramtype build_pod_name: str
        """
        super(BuildResultProperties, self).__init__(**kwargs)
        self.name = name
        self.provisioning_state = None
        self.build_pod_name = build_pod_name
        self.build_stages = None


class UserSourceInfo(msrest.serialization.Model):
    """Source information for a deployment.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: BuildResultUserSourceInfo, UploadedUserSourceInfo.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of the source uploaded.Constant filled by server.
    :vartype type: str
    :ivar version: Version of the source.
    :vartype version: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
    }

    _subtype_map = {
        'type': {'BuildResult': 'BuildResultUserSourceInfo', 'UploadedUserSourceInfo': 'UploadedUserSourceInfo'}
    }

    def __init__(
        self,
        *,
        version: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword version: Version of the source.
        :paramtype version: str
        """
        super(UserSourceInfo, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]
        self.version = version


class BuildResultUserSourceInfo(UserSourceInfo):
    """Reference to a build result.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of the source uploaded.Constant filled by server.
    :vartype type: str
    :ivar version: Version of the source.
    :vartype version: str
    :ivar build_result_id: Resource id of an existing succeeded build result under the same Spring
     instance.
    :vartype build_result_id: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'build_result_id': {'key': 'buildResultId', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        version: Optional[str] = None,
        build_result_id: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword version: Version of the source.
        :paramtype version: str
        :keyword build_result_id: Resource id of an existing succeeded build result under the same
         Spring instance.
        :paramtype build_result_id: str
        """
        super(BuildResultUserSourceInfo, self).__init__(version=version, **kwargs)
        self.type = 'BuildResult'  # type: str
        self.build_result_id = build_result_id


class BuildService(ProxyResource):
    """Build service resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the build resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'BuildServiceProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["BuildServiceProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the build resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceProperties
        """
        super(BuildService, self).__init__(**kwargs)
        self.properties = properties


class BuildServiceAgentPoolProperties(msrest.serialization.Model):
    """Build service agent pool properties.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar provisioning_state: Provisioning state of the build service agent pool.
    :vartype provisioning_state: str
    :ivar pool_size: build service agent pool size properties.
    :vartype pool_size:
     ~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceAgentPoolSizeProperties
    """

    _validation = {
        'provisioning_state': {'readonly': True},
    }

    _attribute_map = {
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'pool_size': {'key': 'poolSize', 'type': 'BuildServiceAgentPoolSizeProperties'},
    }

    def __init__(
        self,
        *,
        pool_size: Optional["BuildServiceAgentPoolSizeProperties"] = None,
        **kwargs
    ):
        """
        :keyword pool_size: build service agent pool size properties.
        :paramtype pool_size:
         ~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceAgentPoolSizeProperties
        """
        super(BuildServiceAgentPoolProperties, self).__init__(**kwargs)
        self.provisioning_state = None
        self.pool_size = pool_size


class BuildServiceAgentPoolResource(ProxyResource):
    """The build service agent pool resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: build service agent pool properties.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceAgentPoolProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'BuildServiceAgentPoolProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["BuildServiceAgentPoolProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: build service agent pool properties.
        :paramtype properties:
         ~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceAgentPoolProperties
        """
        super(BuildServiceAgentPoolResource, self).__init__(**kwargs)
        self.properties = properties


class BuildServiceAgentPoolResourceCollection(msrest.serialization.Model):
    """Object that includes an array of build service agent pool resources and a possible link for next set.

    :ivar value: Collection of build service agent pool resource.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceAgentPoolResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[BuildServiceAgentPoolResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["BuildServiceAgentPoolResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of build service agent pool resource.
        :paramtype value:
         list[~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceAgentPoolResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(BuildServiceAgentPoolResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class BuildServiceAgentPoolSizeProperties(msrest.serialization.Model):
    """Build service agent pool size properties.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: The name of build service agent pool size.
    :vartype name: str
    :ivar cpu: The cpu property of build service agent pool size.
    :vartype cpu: str
    :ivar memory: The memory property of build service agent pool size.
    :vartype memory: str
    """

    _validation = {
        'cpu': {'readonly': True},
        'memory': {'readonly': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'cpu': {'key': 'cpu', 'type': 'str'},
        'memory': {'key': 'memory', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword name: The name of build service agent pool size.
        :paramtype name: str
        """
        super(BuildServiceAgentPoolSizeProperties, self).__init__(**kwargs)
        self.name = name
        self.cpu = None
        self.memory = None


class BuildServiceCollection(msrest.serialization.Model):
    """Object that includes an array of Build service resources and a possible link for next set.

    :ivar value: Collection of Build service resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildService]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[BuildService]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["BuildService"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Build service resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.BuildService]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(BuildServiceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class BuildServiceProperties(msrest.serialization.Model):
    """Build service resource properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar k_pack_version: The installed KPack version in this build service.
    :vartype k_pack_version: str
    :ivar provisioning_state: Provisioning state of the KPack build result. Possible values
     include: "Creating", "Updating", "Succeeded", "Failed", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.BuildServiceProvisioningState
    :ivar resource_requests: The runtime resource configuration of this build service.
    :vartype resource_requests:
     ~azure.mgmt.appplatform.v2022_04_01.models.BuildServicePropertiesResourceRequests
    """

    _validation = {
        'provisioning_state': {'readonly': True},
    }

    _attribute_map = {
        'k_pack_version': {'key': 'kPackVersion', 'type': 'str'},
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'resource_requests': {'key': 'resourceRequests', 'type': 'BuildServicePropertiesResourceRequests'},
    }

    def __init__(
        self,
        *,
        k_pack_version: Optional[str] = None,
        resource_requests: Optional["BuildServicePropertiesResourceRequests"] = None,
        **kwargs
    ):
        """
        :keyword k_pack_version: The installed KPack version in this build service.
        :paramtype k_pack_version: str
        :keyword resource_requests: The runtime resource configuration of this build service.
        :paramtype resource_requests:
         ~azure.mgmt.appplatform.v2022_04_01.models.BuildServicePropertiesResourceRequests
        """
        super(BuildServiceProperties, self).__init__(**kwargs)
        self.k_pack_version = k_pack_version
        self.provisioning_state = None
        self.resource_requests = resource_requests


class BuildServicePropertiesResourceRequests(msrest.serialization.Model):
    """The runtime resource configuration of this build service.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar cpu: vCPU allocated to the entire build service node pool.
    :vartype cpu: str
    :ivar memory: Memory allocated to the entire build service node pool.
    :vartype memory: str
    """

    _validation = {
        'cpu': {'readonly': True},
        'memory': {'readonly': True},
    }

    _attribute_map = {
        'cpu': {'key': 'cpu', 'type': 'str'},
        'memory': {'key': 'memory', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(BuildServicePropertiesResourceRequests, self).__init__(**kwargs)
        self.cpu = None
        self.memory = None


class BuildStageProperties(msrest.serialization.Model):
    """The build stage (init-container and container) resources in build pod.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: The name of this build stage resource.
    :vartype name: str
    :ivar status: The provisioning state of this build stage resource. Possible values include:
     "NotStarted", "Running", "Succeeded", "Failed".
    :vartype status: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.KPackBuildStageProvisioningState
    """

    _validation = {
        'name': {'readonly': True},
        'status': {'readonly': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(BuildStageProperties, self).__init__(**kwargs)
        self.name = None
        self.status = None


class CertificateProperties(msrest.serialization.Model):
    """Certificate resource payload.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: ContentCertificateProperties, KeyVaultCertificateProperties.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. The type of the certificate source.Constant filled by server.
    :vartype type: str
    :ivar thumbprint: The thumbprint of certificate.
    :vartype thumbprint: str
    :ivar issuer: The issuer of certificate.
    :vartype issuer: str
    :ivar issued_date: The issue date of certificate.
    :vartype issued_date: str
    :ivar expiration_date: The expiration date of certificate.
    :vartype expiration_date: str
    :ivar activate_date: The activate date of certificate.
    :vartype activate_date: str
    :ivar subject_name: The subject name of certificate.
    :vartype subject_name: str
    :ivar dns_names: The domain list of certificate.
    :vartype dns_names: list[str]
    """

    _validation = {
        'type': {'required': True},
        'thumbprint': {'readonly': True},
        'issuer': {'readonly': True},
        'issued_date': {'readonly': True},
        'expiration_date': {'readonly': True},
        'activate_date': {'readonly': True},
        'subject_name': {'readonly': True},
        'dns_names': {'readonly': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'thumbprint': {'key': 'thumbprint', 'type': 'str'},
        'issuer': {'key': 'issuer', 'type': 'str'},
        'issued_date': {'key': 'issuedDate', 'type': 'str'},
        'expiration_date': {'key': 'expirationDate', 'type': 'str'},
        'activate_date': {'key': 'activateDate', 'type': 'str'},
        'subject_name': {'key': 'subjectName', 'type': 'str'},
        'dns_names': {'key': 'dnsNames', 'type': '[str]'},
    }

    _subtype_map = {
        'type': {'ContentCertificate': 'ContentCertificateProperties', 'KeyVaultCertificate': 'KeyVaultCertificateProperties'}
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(CertificateProperties, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]
        self.thumbprint = None
        self.issuer = None
        self.issued_date = None
        self.expiration_date = None
        self.activate_date = None
        self.subject_name = None
        self.dns_names = None


class CertificateResource(ProxyResource):
    """Certificate resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the certificate resource payload.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.CertificateProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'CertificateProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["CertificateProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the certificate resource payload.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.CertificateProperties
        """
        super(CertificateResource, self).__init__(**kwargs)
        self.properties = properties


class CertificateResourceCollection(msrest.serialization.Model):
    """Collection compose of certificate resources list and a possible link for next page.

    :ivar value: The certificate resources list.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.CertificateResource]
    :ivar next_link: The link to next page of certificate list.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[CertificateResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["CertificateResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: The certificate resources list.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.CertificateResource]
        :keyword next_link: The link to next page of certificate list.
        :paramtype next_link: str
        """
        super(CertificateResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class CloudErrorBody(msrest.serialization.Model):
    """An error response from the service.

    :ivar code: An identifier for the error. Codes are invariant and are intended to be consumed
     programmatically.
    :vartype code: str
    :ivar message: A message describing the error, intended to be suitable for display in a user
     interface.
    :vartype message: str
    :ivar target: The target of the particular error. For example, the name of the property in
     error.
    :vartype target: str
    :ivar details: A list of additional details about the error.
    :vartype details: list[~azure.mgmt.appplatform.v2022_04_01.models.CloudErrorBody]
    """

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
        'details': {'key': 'details', 'type': '[CloudErrorBody]'},
    }

    def __init__(
        self,
        *,
        code: Optional[str] = None,
        message: Optional[str] = None,
        target: Optional[str] = None,
        details: Optional[List["CloudErrorBody"]] = None,
        **kwargs
    ):
        """
        :keyword code: An identifier for the error. Codes are invariant and are intended to be consumed
         programmatically.
        :paramtype code: str
        :keyword message: A message describing the error, intended to be suitable for display in a user
         interface.
        :paramtype message: str
        :keyword target: The target of the particular error. For example, the name of the property in
         error.
        :paramtype target: str
        :keyword details: A list of additional details about the error.
        :paramtype details: list[~azure.mgmt.appplatform.v2022_04_01.models.CloudErrorBody]
        """
        super(CloudErrorBody, self).__init__(**kwargs)
        self.code = code
        self.message = message
        self.target = target
        self.details = details


class ClusterResourceProperties(msrest.serialization.Model):
    """Service properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar provisioning_state: Provisioning state of the Service. Possible values include:
     "Creating", "Updating", "Starting", "Stopping", "Deleting", "Deleted", "Succeeded", "Failed",
     "Moving", "Moved", "MoveFailed".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.ProvisioningState
    :ivar network_profile: Network profile of the Service.
    :vartype network_profile: ~azure.mgmt.appplatform.v2022_04_01.models.NetworkProfile
    :ivar version: Version of the Service.
    :vartype version: int
    :ivar service_id: ServiceInstanceEntity GUID which uniquely identifies a created resource.
    :vartype service_id: str
    :ivar zone_redundant:
    :vartype zone_redundant: bool
    :ivar fqdn: Fully qualified dns name of the service instance.
    :vartype fqdn: str
    """

    _validation = {
        'provisioning_state': {'readonly': True},
        'version': {'readonly': True},
        'service_id': {'readonly': True},
        'fqdn': {'readonly': True},
    }

    _attribute_map = {
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'network_profile': {'key': 'networkProfile', 'type': 'NetworkProfile'},
        'version': {'key': 'version', 'type': 'int'},
        'service_id': {'key': 'serviceId', 'type': 'str'},
        'zone_redundant': {'key': 'zoneRedundant', 'type': 'bool'},
        'fqdn': {'key': 'fqdn', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        network_profile: Optional["NetworkProfile"] = None,
        zone_redundant: Optional[bool] = False,
        **kwargs
    ):
        """
        :keyword network_profile: Network profile of the Service.
        :paramtype network_profile: ~azure.mgmt.appplatform.v2022_04_01.models.NetworkProfile
        :keyword zone_redundant:
        :paramtype zone_redundant: bool
        """
        super(ClusterResourceProperties, self).__init__(**kwargs)
        self.provisioning_state = None
        self.network_profile = network_profile
        self.version = None
        self.service_id = None
        self.zone_redundant = zone_redundant
        self.fqdn = None


class ConfigServerGitProperty(msrest.serialization.Model):
    """Property of git.

    All required parameters must be populated in order to send to Azure.

    :ivar repositories: Repositories of git.
    :vartype repositories: list[~azure.mgmt.appplatform.v2022_04_01.models.GitPatternRepository]
    :ivar uri: Required. URI of the repository.
    :vartype uri: str
    :ivar label: Label of the repository.
    :vartype label: str
    :ivar search_paths: Searching path of the repository.
    :vartype search_paths: list[str]
    :ivar username: Username of git repository basic auth.
    :vartype username: str
    :ivar password: Password of git repository basic auth.
    :vartype password: str
    :ivar host_key: Public sshKey of git repository.
    :vartype host_key: str
    :ivar host_key_algorithm: SshKey algorithm of git repository.
    :vartype host_key_algorithm: str
    :ivar private_key: Private sshKey algorithm of git repository.
    :vartype private_key: str
    :ivar strict_host_key_checking: Strict host key checking or not.
    :vartype strict_host_key_checking: bool
    """

    _validation = {
        'uri': {'required': True},
    }

    _attribute_map = {
        'repositories': {'key': 'repositories', 'type': '[GitPatternRepository]'},
        'uri': {'key': 'uri', 'type': 'str'},
        'label': {'key': 'label', 'type': 'str'},
        'search_paths': {'key': 'searchPaths', 'type': '[str]'},
        'username': {'key': 'username', 'type': 'str'},
        'password': {'key': 'password', 'type': 'str'},
        'host_key': {'key': 'hostKey', 'type': 'str'},
        'host_key_algorithm': {'key': 'hostKeyAlgorithm', 'type': 'str'},
        'private_key': {'key': 'privateKey', 'type': 'str'},
        'strict_host_key_checking': {'key': 'strictHostKeyChecking', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        uri: str,
        repositories: Optional[List["GitPatternRepository"]] = None,
        label: Optional[str] = None,
        search_paths: Optional[List[str]] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        host_key: Optional[str] = None,
        host_key_algorithm: Optional[str] = None,
        private_key: Optional[str] = None,
        strict_host_key_checking: Optional[bool] = None,
        **kwargs
    ):
        """
        :keyword repositories: Repositories of git.
        :paramtype repositories: list[~azure.mgmt.appplatform.v2022_04_01.models.GitPatternRepository]
        :keyword uri: Required. URI of the repository.
        :paramtype uri: str
        :keyword label: Label of the repository.
        :paramtype label: str
        :keyword search_paths: Searching path of the repository.
        :paramtype search_paths: list[str]
        :keyword username: Username of git repository basic auth.
        :paramtype username: str
        :keyword password: Password of git repository basic auth.
        :paramtype password: str
        :keyword host_key: Public sshKey of git repository.
        :paramtype host_key: str
        :keyword host_key_algorithm: SshKey algorithm of git repository.
        :paramtype host_key_algorithm: str
        :keyword private_key: Private sshKey algorithm of git repository.
        :paramtype private_key: str
        :keyword strict_host_key_checking: Strict host key checking or not.
        :paramtype strict_host_key_checking: bool
        """
        super(ConfigServerGitProperty, self).__init__(**kwargs)
        self.repositories = repositories
        self.uri = uri
        self.label = label
        self.search_paths = search_paths
        self.username = username
        self.password = password
        self.host_key = host_key
        self.host_key_algorithm = host_key_algorithm
        self.private_key = private_key
        self.strict_host_key_checking = strict_host_key_checking


class ConfigServerProperties(msrest.serialization.Model):
    """Config server git properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar provisioning_state: State of the config server. Possible values include: "NotAvailable",
     "Deleted", "Failed", "Succeeded", "Updating".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerState
    :ivar error: Error when apply config server settings.
    :vartype error: ~azure.mgmt.appplatform.v2022_04_01.models.Error
    :ivar config_server: Settings of config server.
    :vartype config_server: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerSettings
    """

    _validation = {
        'provisioning_state': {'readonly': True},
    }

    _attribute_map = {
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'error': {'key': 'error', 'type': 'Error'},
        'config_server': {'key': 'configServer', 'type': 'ConfigServerSettings'},
    }

    def __init__(
        self,
        *,
        error: Optional["Error"] = None,
        config_server: Optional["ConfigServerSettings"] = None,
        **kwargs
    ):
        """
        :keyword error: Error when apply config server settings.
        :paramtype error: ~azure.mgmt.appplatform.v2022_04_01.models.Error
        :keyword config_server: Settings of config server.
        :paramtype config_server: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerSettings
        """
        super(ConfigServerProperties, self).__init__(**kwargs)
        self.provisioning_state = None
        self.error = error
        self.config_server = config_server


class ConfigServerResource(ProxyResource):
    """Config Server resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the Config Server resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'ConfigServerProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["ConfigServerProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the Config Server resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerProperties
        """
        super(ConfigServerResource, self).__init__(**kwargs)
        self.properties = properties


class ConfigServerSettings(msrest.serialization.Model):
    """The settings of config server.

    :ivar git_property: Property of git environment.
    :vartype git_property: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerGitProperty
    """

    _attribute_map = {
        'git_property': {'key': 'gitProperty', 'type': 'ConfigServerGitProperty'},
    }

    def __init__(
        self,
        *,
        git_property: Optional["ConfigServerGitProperty"] = None,
        **kwargs
    ):
        """
        :keyword git_property: Property of git environment.
        :paramtype git_property: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerGitProperty
        """
        super(ConfigServerSettings, self).__init__(**kwargs)
        self.git_property = git_property


class ConfigServerSettingsErrorRecord(msrest.serialization.Model):
    """Error record of the config server settings.

    :ivar name: The name of the config server settings error record.
    :vartype name: str
    :ivar uri: The uri of the config server settings error record.
    :vartype uri: str
    :ivar messages: The detail error messages of the record.
    :vartype messages: list[str]
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'uri': {'key': 'uri', 'type': 'str'},
        'messages': {'key': 'messages', 'type': '[str]'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        uri: Optional[str] = None,
        messages: Optional[List[str]] = None,
        **kwargs
    ):
        """
        :keyword name: The name of the config server settings error record.
        :paramtype name: str
        :keyword uri: The uri of the config server settings error record.
        :paramtype uri: str
        :keyword messages: The detail error messages of the record.
        :paramtype messages: list[str]
        """
        super(ConfigServerSettingsErrorRecord, self).__init__(**kwargs)
        self.name = name
        self.uri = uri
        self.messages = messages


class ConfigServerSettingsValidateResult(msrest.serialization.Model):
    """Validation result for config server settings.

    :ivar is_valid: Indicate if the config server settings are valid.
    :vartype is_valid: bool
    :ivar details: The detail validation results.
    :vartype details:
     list[~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerSettingsErrorRecord]
    """

    _attribute_map = {
        'is_valid': {'key': 'isValid', 'type': 'bool'},
        'details': {'key': 'details', 'type': '[ConfigServerSettingsErrorRecord]'},
    }

    def __init__(
        self,
        *,
        is_valid: Optional[bool] = None,
        details: Optional[List["ConfigServerSettingsErrorRecord"]] = None,
        **kwargs
    ):
        """
        :keyword is_valid: Indicate if the config server settings are valid.
        :paramtype is_valid: bool
        :keyword details: The detail validation results.
        :paramtype details:
         list[~azure.mgmt.appplatform.v2022_04_01.models.ConfigServerSettingsErrorRecord]
        """
        super(ConfigServerSettingsValidateResult, self).__init__(**kwargs)
        self.is_valid = is_valid
        self.details = details


class ConfigurationServiceGitProperty(msrest.serialization.Model):
    """Property of git environment.

    :ivar repositories: Repositories of Application Configuration Service git property.
    :vartype repositories:
     list[~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceGitRepository]
    """

    _attribute_map = {
        'repositories': {'key': 'repositories', 'type': '[ConfigurationServiceGitRepository]'},
    }

    def __init__(
        self,
        *,
        repositories: Optional[List["ConfigurationServiceGitRepository"]] = None,
        **kwargs
    ):
        """
        :keyword repositories: Repositories of Application Configuration Service git property.
        :paramtype repositories:
         list[~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceGitRepository]
        """
        super(ConfigurationServiceGitProperty, self).__init__(**kwargs)
        self.repositories = repositories


class ConfigurationServiceGitPropertyValidateResult(msrest.serialization.Model):
    """Validation result for configuration service settings.

    :ivar is_valid: Indicate if the configuration service settings are valid.
    :vartype is_valid: bool
    :ivar git_repos_validation_result: The detail validation results.
    :vartype git_repos_validation_result:
     list[~azure.mgmt.appplatform.v2022_04_01.models.ValidationMessages]
    """

    _attribute_map = {
        'is_valid': {'key': 'isValid', 'type': 'bool'},
        'git_repos_validation_result': {'key': 'gitReposValidationResult', 'type': '[ValidationMessages]'},
    }

    def __init__(
        self,
        *,
        is_valid: Optional[bool] = None,
        git_repos_validation_result: Optional[List["ValidationMessages"]] = None,
        **kwargs
    ):
        """
        :keyword is_valid: Indicate if the configuration service settings are valid.
        :paramtype is_valid: bool
        :keyword git_repos_validation_result: The detail validation results.
        :paramtype git_repos_validation_result:
         list[~azure.mgmt.appplatform.v2022_04_01.models.ValidationMessages]
        """
        super(ConfigurationServiceGitPropertyValidateResult, self).__init__(**kwargs)
        self.is_valid = is_valid
        self.git_repos_validation_result = git_repos_validation_result


class ConfigurationServiceGitRepository(msrest.serialization.Model):
    """Git repository property payload for Application Configuration Service.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required. Name of the repository.
    :vartype name: str
    :ivar patterns: Required. Collection of patterns of the repository.
    :vartype patterns: list[str]
    :ivar uri: Required. URI of the repository.
    :vartype uri: str
    :ivar label: Required. Label of the repository.
    :vartype label: str
    :ivar search_paths: Searching path of the repository.
    :vartype search_paths: list[str]
    :ivar username: Username of git repository basic auth.
    :vartype username: str
    :ivar password: Password of git repository basic auth.
    :vartype password: str
    :ivar host_key: Public sshKey of git repository.
    :vartype host_key: str
    :ivar host_key_algorithm: SshKey algorithm of git repository.
    :vartype host_key_algorithm: str
    :ivar private_key: Private sshKey algorithm of git repository.
    :vartype private_key: str
    :ivar strict_host_key_checking: Strict host key checking or not.
    :vartype strict_host_key_checking: bool
    """

    _validation = {
        'name': {'required': True},
        'patterns': {'required': True},
        'uri': {'required': True},
        'label': {'required': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'patterns': {'key': 'patterns', 'type': '[str]'},
        'uri': {'key': 'uri', 'type': 'str'},
        'label': {'key': 'label', 'type': 'str'},
        'search_paths': {'key': 'searchPaths', 'type': '[str]'},
        'username': {'key': 'username', 'type': 'str'},
        'password': {'key': 'password', 'type': 'str'},
        'host_key': {'key': 'hostKey', 'type': 'str'},
        'host_key_algorithm': {'key': 'hostKeyAlgorithm', 'type': 'str'},
        'private_key': {'key': 'privateKey', 'type': 'str'},
        'strict_host_key_checking': {'key': 'strictHostKeyChecking', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        name: str,
        patterns: List[str],
        uri: str,
        label: str,
        search_paths: Optional[List[str]] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        host_key: Optional[str] = None,
        host_key_algorithm: Optional[str] = None,
        private_key: Optional[str] = None,
        strict_host_key_checking: Optional[bool] = None,
        **kwargs
    ):
        """
        :keyword name: Required. Name of the repository.
        :paramtype name: str
        :keyword patterns: Required. Collection of patterns of the repository.
        :paramtype patterns: list[str]
        :keyword uri: Required. URI of the repository.
        :paramtype uri: str
        :keyword label: Required. Label of the repository.
        :paramtype label: str
        :keyword search_paths: Searching path of the repository.
        :paramtype search_paths: list[str]
        :keyword username: Username of git repository basic auth.
        :paramtype username: str
        :keyword password: Password of git repository basic auth.
        :paramtype password: str
        :keyword host_key: Public sshKey of git repository.
        :paramtype host_key: str
        :keyword host_key_algorithm: SshKey algorithm of git repository.
        :paramtype host_key_algorithm: str
        :keyword private_key: Private sshKey algorithm of git repository.
        :paramtype private_key: str
        :keyword strict_host_key_checking: Strict host key checking or not.
        :paramtype strict_host_key_checking: bool
        """
        super(ConfigurationServiceGitRepository, self).__init__(**kwargs)
        self.name = name
        self.patterns = patterns
        self.uri = uri
        self.label = label
        self.search_paths = search_paths
        self.username = username
        self.password = password
        self.host_key = host_key
        self.host_key_algorithm = host_key_algorithm
        self.private_key = private_key
        self.strict_host_key_checking = strict_host_key_checking


class ConfigurationServiceInstance(msrest.serialization.Model):
    """Collection of instances belong to the Application Configuration Service.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: Name of the Application Configuration Service instance.
    :vartype name: str
    :ivar status: Status of the Application Configuration Service instance.
    :vartype status: str
    """

    _validation = {
        'name': {'readonly': True},
        'status': {'readonly': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(ConfigurationServiceInstance, self).__init__(**kwargs)
        self.name = None
        self.status = None


class ConfigurationServiceProperties(msrest.serialization.Model):
    """Application Configuration Service properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar provisioning_state: State of the Application Configuration Service. Possible values
     include: "Creating", "Updating", "Succeeded", "Failed", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceProvisioningState
    :ivar resource_requests: The requested resource quantity for required CPU and Memory.
    :vartype resource_requests:
     ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceResourceRequests
    :ivar instances: Collection of instances belong to Application Configuration Service.
    :vartype instances:
     list[~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceInstance]
    :ivar settings: The settings of Application Configuration Service.
    :vartype settings: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceSettings
    """

    _validation = {
        'provisioning_state': {'readonly': True},
        'resource_requests': {'readonly': True},
        'instances': {'readonly': True},
    }

    _attribute_map = {
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'resource_requests': {'key': 'resourceRequests', 'type': 'ConfigurationServiceResourceRequests'},
        'instances': {'key': 'instances', 'type': '[ConfigurationServiceInstance]'},
        'settings': {'key': 'settings', 'type': 'ConfigurationServiceSettings'},
    }

    def __init__(
        self,
        *,
        settings: Optional["ConfigurationServiceSettings"] = None,
        **kwargs
    ):
        """
        :keyword settings: The settings of Application Configuration Service.
        :paramtype settings: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceSettings
        """
        super(ConfigurationServiceProperties, self).__init__(**kwargs)
        self.provisioning_state = None
        self.resource_requests = None
        self.instances = None
        self.settings = settings


class ConfigurationServiceResource(ProxyResource):
    """Application Configuration Service resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Application Configuration Service properties payload.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'ConfigurationServiceProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["ConfigurationServiceProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Application Configuration Service properties payload.
        :paramtype properties:
         ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceProperties
        """
        super(ConfigurationServiceResource, self).__init__(**kwargs)
        self.properties = properties


class ConfigurationServiceResourceCollection(msrest.serialization.Model):
    """Object that includes an array of configuration service resources and a possible link for next set.

    :ivar value: Collection of configuration service resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[ConfigurationServiceResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["ConfigurationServiceResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of configuration service resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(ConfigurationServiceResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ConfigurationServiceResourceRequests(msrest.serialization.Model):
    """Resource request payload of Application Configuration Service.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar cpu: Cpu allocated to each Application Configuration Service instance.
    :vartype cpu: str
    :ivar memory: Memory allocated to each Application Configuration Service instance.
    :vartype memory: str
    :ivar instance_count: Instance count of the Application Configuration Service.
    :vartype instance_count: int
    """

    _validation = {
        'cpu': {'readonly': True},
        'memory': {'readonly': True},
        'instance_count': {'readonly': True},
    }

    _attribute_map = {
        'cpu': {'key': 'cpu', 'type': 'str'},
        'memory': {'key': 'memory', 'type': 'str'},
        'instance_count': {'key': 'instanceCount', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(ConfigurationServiceResourceRequests, self).__init__(**kwargs)
        self.cpu = None
        self.memory = None
        self.instance_count = None


class ConfigurationServiceSettings(msrest.serialization.Model):
    """The settings of Application Configuration Service.

    :ivar git_property: Property of git environment.
    :vartype git_property:
     ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceGitProperty
    """

    _attribute_map = {
        'git_property': {'key': 'gitProperty', 'type': 'ConfigurationServiceGitProperty'},
    }

    def __init__(
        self,
        *,
        git_property: Optional["ConfigurationServiceGitProperty"] = None,
        **kwargs
    ):
        """
        :keyword git_property: Property of git environment.
        :paramtype git_property:
         ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceGitProperty
        """
        super(ConfigurationServiceSettings, self).__init__(**kwargs)
        self.git_property = git_property


class ConfigurationServiceSettingsValidateResult(msrest.serialization.Model):
    """Validation result for configuration service settings.

    :ivar git_property_validation_result: Validation result for configuration service settings.
    :vartype git_property_validation_result:
     ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceGitPropertyValidateResult
    """

    _attribute_map = {
        'git_property_validation_result': {'key': 'gitPropertyValidationResult', 'type': 'ConfigurationServiceGitPropertyValidateResult'},
    }

    def __init__(
        self,
        *,
        git_property_validation_result: Optional["ConfigurationServiceGitPropertyValidateResult"] = None,
        **kwargs
    ):
        """
        :keyword git_property_validation_result: Validation result for configuration service settings.
        :paramtype git_property_validation_result:
         ~azure.mgmt.appplatform.v2022_04_01.models.ConfigurationServiceGitPropertyValidateResult
        """
        super(ConfigurationServiceSettingsValidateResult, self).__init__(**kwargs)
        self.git_property_validation_result = git_property_validation_result


class ContentCertificateProperties(CertificateProperties):
    """Properties of certificate imported from key vault.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. The type of the certificate source.Constant filled by server.
    :vartype type: str
    :ivar thumbprint: The thumbprint of certificate.
    :vartype thumbprint: str
    :ivar issuer: The issuer of certificate.
    :vartype issuer: str
    :ivar issued_date: The issue date of certificate.
    :vartype issued_date: str
    :ivar expiration_date: The expiration date of certificate.
    :vartype expiration_date: str
    :ivar activate_date: The activate date of certificate.
    :vartype activate_date: str
    :ivar subject_name: The subject name of certificate.
    :vartype subject_name: str
    :ivar dns_names: The domain list of certificate.
    :vartype dns_names: list[str]
    :ivar content: The content of uploaded certificate.
    :vartype content: str
    """

    _validation = {
        'type': {'required': True},
        'thumbprint': {'readonly': True},
        'issuer': {'readonly': True},
        'issued_date': {'readonly': True},
        'expiration_date': {'readonly': True},
        'activate_date': {'readonly': True},
        'subject_name': {'readonly': True},
        'dns_names': {'readonly': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'thumbprint': {'key': 'thumbprint', 'type': 'str'},
        'issuer': {'key': 'issuer', 'type': 'str'},
        'issued_date': {'key': 'issuedDate', 'type': 'str'},
        'expiration_date': {'key': 'expirationDate', 'type': 'str'},
        'activate_date': {'key': 'activateDate', 'type': 'str'},
        'subject_name': {'key': 'subjectName', 'type': 'str'},
        'dns_names': {'key': 'dnsNames', 'type': '[str]'},
        'content': {'key': 'content', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        content: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword content: The content of uploaded certificate.
        :paramtype content: str
        """
        super(ContentCertificateProperties, self).__init__(**kwargs)
        self.type = 'ContentCertificate'  # type: str
        self.content = content


class CustomDomainProperties(msrest.serialization.Model):
    """Custom domain of app resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar thumbprint: The thumbprint of bound certificate.
    :vartype thumbprint: str
    :ivar app_name: The app name of domain.
    :vartype app_name: str
    :ivar cert_name: The bound certificate name of domain.
    :vartype cert_name: str
    """

    _validation = {
        'app_name': {'readonly': True},
    }

    _attribute_map = {
        'thumbprint': {'key': 'thumbprint', 'type': 'str'},
        'app_name': {'key': 'appName', 'type': 'str'},
        'cert_name': {'key': 'certName', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        thumbprint: Optional[str] = None,
        cert_name: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword thumbprint: The thumbprint of bound certificate.
        :paramtype thumbprint: str
        :keyword cert_name: The bound certificate name of domain.
        :paramtype cert_name: str
        """
        super(CustomDomainProperties, self).__init__(**kwargs)
        self.thumbprint = thumbprint
        self.app_name = None
        self.cert_name = cert_name


class CustomDomainResource(ProxyResource):
    """Custom domain resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the custom domain resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.CustomDomainProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'CustomDomainProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["CustomDomainProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the custom domain resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.CustomDomainProperties
        """
        super(CustomDomainResource, self).__init__(**kwargs)
        self.properties = properties


class CustomDomainResourceCollection(msrest.serialization.Model):
    """Collection compose of a custom domain resources list and a possible link for next page.

    :ivar value: The custom domain resources list.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.CustomDomainResource]
    :ivar next_link: The link to next page of custom domain list.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[CustomDomainResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["CustomDomainResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: The custom domain resources list.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.CustomDomainResource]
        :keyword next_link: The link to next page of custom domain list.
        :paramtype next_link: str
        """
        super(CustomDomainResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class CustomDomainValidatePayload(msrest.serialization.Model):
    """Custom domain validate payload.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required. Name to be validated.
    :vartype name: str
    """

    _validation = {
        'name': {'required': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: str,
        **kwargs
    ):
        """
        :keyword name: Required. Name to be validated.
        :paramtype name: str
        """
        super(CustomDomainValidatePayload, self).__init__(**kwargs)
        self.name = name


class CustomDomainValidateResult(msrest.serialization.Model):
    """Validation result for custom domain.

    :ivar is_valid: Indicates if domain name is valid.
    :vartype is_valid: bool
    :ivar message: Message of why domain name is invalid.
    :vartype message: str
    """

    _attribute_map = {
        'is_valid': {'key': 'isValid', 'type': 'bool'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        is_valid: Optional[bool] = None,
        message: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword is_valid: Indicates if domain name is valid.
        :paramtype is_valid: bool
        :keyword message: Message of why domain name is invalid.
        :paramtype message: str
        """
        super(CustomDomainValidateResult, self).__init__(**kwargs)
        self.is_valid = is_valid
        self.message = message


class DeploymentInstance(msrest.serialization.Model):
    """Deployment instance payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: Name of the deployment instance.
    :vartype name: str
    :ivar status: Status of the deployment instance.
    :vartype status: str
    :ivar reason: Failed reason of the deployment instance.
    :vartype reason: str
    :ivar discovery_status: Discovery status of the deployment instance.
    :vartype discovery_status: str
    :ivar start_time: Start time of the deployment instance.
    :vartype start_time: str
    :ivar zone: Availability zone information of the deployment instance.
    :vartype zone: str
    """

    _validation = {
        'name': {'readonly': True},
        'status': {'readonly': True},
        'reason': {'readonly': True},
        'discovery_status': {'readonly': True},
        'start_time': {'readonly': True},
        'zone': {'readonly': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'reason': {'key': 'reason', 'type': 'str'},
        'discovery_status': {'key': 'discoveryStatus', 'type': 'str'},
        'start_time': {'key': 'startTime', 'type': 'str'},
        'zone': {'key': 'zone', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(DeploymentInstance, self).__init__(**kwargs)
        self.name = None
        self.status = None
        self.reason = None
        self.discovery_status = None
        self.start_time = None
        self.zone = None


class DeploymentResource(ProxyResource):
    """Deployment resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the Deployment resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.DeploymentResourceProperties
    :ivar sku: Sku of the Deployment resource.
    :vartype sku: ~azure.mgmt.appplatform.v2022_04_01.models.Sku
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'DeploymentResourceProperties'},
        'sku': {'key': 'sku', 'type': 'Sku'},
    }

    def __init__(
        self,
        *,
        properties: Optional["DeploymentResourceProperties"] = None,
        sku: Optional["Sku"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the Deployment resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.DeploymentResourceProperties
        :keyword sku: Sku of the Deployment resource.
        :paramtype sku: ~azure.mgmt.appplatform.v2022_04_01.models.Sku
        """
        super(DeploymentResource, self).__init__(**kwargs)
        self.properties = properties
        self.sku = sku


class DeploymentResourceCollection(msrest.serialization.Model):
    """Object that includes an array of App resources and a possible link for next set.

    :ivar value: Collection of Deployment resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.DeploymentResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[DeploymentResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["DeploymentResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Deployment resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.DeploymentResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(DeploymentResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class DeploymentResourceProperties(msrest.serialization.Model):
    """Deployment resource properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar source: Uploaded source information of the deployment.
    :vartype source: ~azure.mgmt.appplatform.v2022_04_01.models.UserSourceInfo
    :ivar deployment_settings: Deployment settings of the Deployment.
    :vartype deployment_settings: ~azure.mgmt.appplatform.v2022_04_01.models.DeploymentSettings
    :ivar provisioning_state: Provisioning state of the Deployment. Possible values include:
     "Creating", "Updating", "Succeeded", "Failed".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.DeploymentResourceProvisioningState
    :ivar status: Status of the Deployment. Possible values include: "Stopped", "Running".
    :vartype status: str or ~azure.mgmt.appplatform.v2022_04_01.models.DeploymentResourceStatus
    :ivar active: Indicates whether the Deployment is active.
    :vartype active: bool
    :ivar instances: Collection of instances belong to the Deployment.
    :vartype instances: list[~azure.mgmt.appplatform.v2022_04_01.models.DeploymentInstance]
    """

    _validation = {
        'provisioning_state': {'readonly': True},
        'status': {'readonly': True},
        'instances': {'readonly': True},
    }

    _attribute_map = {
        'source': {'key': 'source', 'type': 'UserSourceInfo'},
        'deployment_settings': {'key': 'deploymentSettings', 'type': 'DeploymentSettings'},
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'active': {'key': 'active', 'type': 'bool'},
        'instances': {'key': 'instances', 'type': '[DeploymentInstance]'},
    }

    def __init__(
        self,
        *,
        source: Optional["UserSourceInfo"] = None,
        deployment_settings: Optional["DeploymentSettings"] = None,
        active: Optional[bool] = None,
        **kwargs
    ):
        """
        :keyword source: Uploaded source information of the deployment.
        :paramtype source: ~azure.mgmt.appplatform.v2022_04_01.models.UserSourceInfo
        :keyword deployment_settings: Deployment settings of the Deployment.
        :paramtype deployment_settings: ~azure.mgmt.appplatform.v2022_04_01.models.DeploymentSettings
        :keyword active: Indicates whether the Deployment is active.
        :paramtype active: bool
        """
        super(DeploymentResourceProperties, self).__init__(**kwargs)
        self.source = source
        self.deployment_settings = deployment_settings
        self.provisioning_state = None
        self.status = None
        self.active = active
        self.instances = None


class DeploymentSettings(msrest.serialization.Model):
    """Deployment settings payload.

    :ivar resource_requests: The requested resource quantity for required CPU and Memory. It is
     recommended that using this field to represent the required CPU and Memory, the old field cpu
     and memoryInGB will be deprecated later.
    :vartype resource_requests: ~azure.mgmt.appplatform.v2022_04_01.models.ResourceRequests
    :ivar environment_variables: Collection of environment variables.
    :vartype environment_variables: dict[str, str]
    :ivar addon_configs: Collection of addons.
    :vartype addon_configs: dict[str, dict[str, any]]
    """

    _attribute_map = {
        'resource_requests': {'key': 'resourceRequests', 'type': 'ResourceRequests'},
        'environment_variables': {'key': 'environmentVariables', 'type': '{str}'},
        'addon_configs': {'key': 'addonConfigs', 'type': '{{object}}'},
    }

    def __init__(
        self,
        *,
        resource_requests: Optional["ResourceRequests"] = None,
        environment_variables: Optional[Dict[str, str]] = None,
        addon_configs: Optional[Dict[str, Dict[str, Any]]] = None,
        **kwargs
    ):
        """
        :keyword resource_requests: The requested resource quantity for required CPU and Memory. It is
         recommended that using this field to represent the required CPU and Memory, the old field cpu
         and memoryInGB will be deprecated later.
        :paramtype resource_requests: ~azure.mgmt.appplatform.v2022_04_01.models.ResourceRequests
        :keyword environment_variables: Collection of environment variables.
        :paramtype environment_variables: dict[str, str]
        :keyword addon_configs: Collection of addons.
        :paramtype addon_configs: dict[str, dict[str, any]]
        """
        super(DeploymentSettings, self).__init__(**kwargs)
        self.resource_requests = resource_requests
        self.environment_variables = environment_variables
        self.addon_configs = addon_configs


class DiagnosticParameters(msrest.serialization.Model):
    """Diagnostic parameters of diagnostic operations.

    :ivar app_instance: App instance name.
    :vartype app_instance: str
    :ivar file_path: Your target file path in your own BYOS.
    :vartype file_path: str
    :ivar duration: Duration of your JFR. 1 min can be represented by 1m or 60s.
    :vartype duration: str
    """

    _attribute_map = {
        'app_instance': {'key': 'appInstance', 'type': 'str'},
        'file_path': {'key': 'filePath', 'type': 'str'},
        'duration': {'key': 'duration', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        app_instance: Optional[str] = None,
        file_path: Optional[str] = None,
        duration: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword app_instance: App instance name.
        :paramtype app_instance: str
        :keyword file_path: Your target file path in your own BYOS.
        :paramtype file_path: str
        :keyword duration: Duration of your JFR. 1 min can be represented by 1m or 60s.
        :paramtype duration: str
        """
        super(DiagnosticParameters, self).__init__(**kwargs)
        self.app_instance = app_instance
        self.file_path = file_path
        self.duration = duration


class Error(msrest.serialization.Model):
    """The error code compose of code and message.

    :ivar code: The code of error.
    :vartype code: str
    :ivar message: The message of error.
    :vartype message: str
    """

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        code: Optional[str] = None,
        message: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword code: The code of error.
        :paramtype code: str
        :keyword message: The message of error.
        :paramtype message: str
        """
        super(Error, self).__init__(**kwargs)
        self.code = code
        self.message = message


class GitPatternRepository(msrest.serialization.Model):
    """Git repository property payload for config server.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required. Name of the repository.
    :vartype name: str
    :ivar pattern: Collection of pattern of the repository.
    :vartype pattern: list[str]
    :ivar uri: Required. URI of the repository.
    :vartype uri: str
    :ivar label: Label of the repository.
    :vartype label: str
    :ivar search_paths: Searching path of the repository.
    :vartype search_paths: list[str]
    :ivar username: Username of git repository basic auth.
    :vartype username: str
    :ivar password: Password of git repository basic auth.
    :vartype password: str
    :ivar host_key: Public sshKey of git repository.
    :vartype host_key: str
    :ivar host_key_algorithm: SshKey algorithm of git repository.
    :vartype host_key_algorithm: str
    :ivar private_key: Private sshKey algorithm of git repository.
    :vartype private_key: str
    :ivar strict_host_key_checking: Strict host key checking or not.
    :vartype strict_host_key_checking: bool
    """

    _validation = {
        'name': {'required': True},
        'uri': {'required': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'pattern': {'key': 'pattern', 'type': '[str]'},
        'uri': {'key': 'uri', 'type': 'str'},
        'label': {'key': 'label', 'type': 'str'},
        'search_paths': {'key': 'searchPaths', 'type': '[str]'},
        'username': {'key': 'username', 'type': 'str'},
        'password': {'key': 'password', 'type': 'str'},
        'host_key': {'key': 'hostKey', 'type': 'str'},
        'host_key_algorithm': {'key': 'hostKeyAlgorithm', 'type': 'str'},
        'private_key': {'key': 'privateKey', 'type': 'str'},
        'strict_host_key_checking': {'key': 'strictHostKeyChecking', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        name: str,
        uri: str,
        pattern: Optional[List[str]] = None,
        label: Optional[str] = None,
        search_paths: Optional[List[str]] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
        host_key: Optional[str] = None,
        host_key_algorithm: Optional[str] = None,
        private_key: Optional[str] = None,
        strict_host_key_checking: Optional[bool] = None,
        **kwargs
    ):
        """
        :keyword name: Required. Name of the repository.
        :paramtype name: str
        :keyword pattern: Collection of pattern of the repository.
        :paramtype pattern: list[str]
        :keyword uri: Required. URI of the repository.
        :paramtype uri: str
        :keyword label: Label of the repository.
        :paramtype label: str
        :keyword search_paths: Searching path of the repository.
        :paramtype search_paths: list[str]
        :keyword username: Username of git repository basic auth.
        :paramtype username: str
        :keyword password: Password of git repository basic auth.
        :paramtype password: str
        :keyword host_key: Public sshKey of git repository.
        :paramtype host_key: str
        :keyword host_key_algorithm: SshKey algorithm of git repository.
        :paramtype host_key_algorithm: str
        :keyword private_key: Private sshKey algorithm of git repository.
        :paramtype private_key: str
        :keyword strict_host_key_checking: Strict host key checking or not.
        :paramtype strict_host_key_checking: bool
        """
        super(GitPatternRepository, self).__init__(**kwargs)
        self.name = name
        self.pattern = pattern
        self.uri = uri
        self.label = label
        self.search_paths = search_paths
        self.username = username
        self.password = password
        self.host_key = host_key
        self.host_key_algorithm = host_key_algorithm
        self.private_key = private_key
        self.strict_host_key_checking = strict_host_key_checking


class UploadedUserSourceInfo(UserSourceInfo):
    """Source with uploaded location.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: JarUploadedUserSourceInfo, NetCoreZipUploadedUserSourceInfo, SourceUploadedUserSourceInfo.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of the source uploaded.Constant filled by server.
    :vartype type: str
    :ivar version: Version of the source.
    :vartype version: str
    :ivar relative_path: Relative path of the storage which stores the source.
    :vartype relative_path: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'relative_path': {'key': 'relativePath', 'type': 'str'},
    }

    _subtype_map = {
        'type': {'Jar': 'JarUploadedUserSourceInfo', 'NetCoreZip': 'NetCoreZipUploadedUserSourceInfo', 'Source': 'SourceUploadedUserSourceInfo'}
    }

    def __init__(
        self,
        *,
        version: Optional[str] = None,
        relative_path: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword version: Version of the source.
        :paramtype version: str
        :keyword relative_path: Relative path of the storage which stores the source.
        :paramtype relative_path: str
        """
        super(UploadedUserSourceInfo, self).__init__(version=version, **kwargs)
        self.type = 'UploadedUserSourceInfo'  # type: str
        self.relative_path = relative_path


class JarUploadedUserSourceInfo(UploadedUserSourceInfo):
    """Uploaded Jar binary for a deployment.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of the source uploaded.Constant filled by server.
    :vartype type: str
    :ivar version: Version of the source.
    :vartype version: str
    :ivar relative_path: Relative path of the storage which stores the source.
    :vartype relative_path: str
    :ivar runtime_version: Runtime version of the Jar file.
    :vartype runtime_version: str
    :ivar jvm_options: JVM parameter.
    :vartype jvm_options: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'relative_path': {'key': 'relativePath', 'type': 'str'},
        'runtime_version': {'key': 'runtimeVersion', 'type': 'str'},
        'jvm_options': {'key': 'jvmOptions', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        version: Optional[str] = None,
        relative_path: Optional[str] = None,
        runtime_version: Optional[str] = None,
        jvm_options: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword version: Version of the source.
        :paramtype version: str
        :keyword relative_path: Relative path of the storage which stores the source.
        :paramtype relative_path: str
        :keyword runtime_version: Runtime version of the Jar file.
        :paramtype runtime_version: str
        :keyword jvm_options: JVM parameter.
        :paramtype jvm_options: str
        """
        super(JarUploadedUserSourceInfo, self).__init__(version=version, relative_path=relative_path, **kwargs)
        self.type = 'Jar'  # type: str
        self.runtime_version = runtime_version
        self.jvm_options = jvm_options


class KeyVaultCertificateProperties(CertificateProperties):
    """Properties of certificate imported from key vault.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. The type of the certificate source.Constant filled by server.
    :vartype type: str
    :ivar thumbprint: The thumbprint of certificate.
    :vartype thumbprint: str
    :ivar issuer: The issuer of certificate.
    :vartype issuer: str
    :ivar issued_date: The issue date of certificate.
    :vartype issued_date: str
    :ivar expiration_date: The expiration date of certificate.
    :vartype expiration_date: str
    :ivar activate_date: The activate date of certificate.
    :vartype activate_date: str
    :ivar subject_name: The subject name of certificate.
    :vartype subject_name: str
    :ivar dns_names: The domain list of certificate.
    :vartype dns_names: list[str]
    :ivar vault_uri: Required. The vault uri of user key vault.
    :vartype vault_uri: str
    :ivar key_vault_cert_name: Required. The certificate name of key vault.
    :vartype key_vault_cert_name: str
    :ivar cert_version: The certificate version of key vault.
    :vartype cert_version: str
    :ivar exclude_private_key: Optional. If set to true, it will not import private key from key
     vault.
    :vartype exclude_private_key: bool
    """

    _validation = {
        'type': {'required': True},
        'thumbprint': {'readonly': True},
        'issuer': {'readonly': True},
        'issued_date': {'readonly': True},
        'expiration_date': {'readonly': True},
        'activate_date': {'readonly': True},
        'subject_name': {'readonly': True},
        'dns_names': {'readonly': True},
        'vault_uri': {'required': True},
        'key_vault_cert_name': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'thumbprint': {'key': 'thumbprint', 'type': 'str'},
        'issuer': {'key': 'issuer', 'type': 'str'},
        'issued_date': {'key': 'issuedDate', 'type': 'str'},
        'expiration_date': {'key': 'expirationDate', 'type': 'str'},
        'activate_date': {'key': 'activateDate', 'type': 'str'},
        'subject_name': {'key': 'subjectName', 'type': 'str'},
        'dns_names': {'key': 'dnsNames', 'type': '[str]'},
        'vault_uri': {'key': 'vaultUri', 'type': 'str'},
        'key_vault_cert_name': {'key': 'keyVaultCertName', 'type': 'str'},
        'cert_version': {'key': 'certVersion', 'type': 'str'},
        'exclude_private_key': {'key': 'excludePrivateKey', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        vault_uri: str,
        key_vault_cert_name: str,
        cert_version: Optional[str] = None,
        exclude_private_key: Optional[bool] = False,
        **kwargs
    ):
        """
        :keyword vault_uri: Required. The vault uri of user key vault.
        :paramtype vault_uri: str
        :keyword key_vault_cert_name: Required. The certificate name of key vault.
        :paramtype key_vault_cert_name: str
        :keyword cert_version: The certificate version of key vault.
        :paramtype cert_version: str
        :keyword exclude_private_key: Optional. If set to true, it will not import private key from key
         vault.
        :paramtype exclude_private_key: bool
        """
        super(KeyVaultCertificateProperties, self).__init__(**kwargs)
        self.type = 'KeyVaultCertificate'  # type: str
        self.vault_uri = vault_uri
        self.key_vault_cert_name = key_vault_cert_name
        self.cert_version = cert_version
        self.exclude_private_key = exclude_private_key


class LoadedCertificate(msrest.serialization.Model):
    """Loaded certificate payload.

    All required parameters must be populated in order to send to Azure.

    :ivar resource_id: Required. Resource Id of loaded certificate.
    :vartype resource_id: str
    :ivar load_trust_store: Indicate whether the certificate will be loaded into default trust
     store, only work for Java runtime.
    :vartype load_trust_store: bool
    """

    _validation = {
        'resource_id': {'required': True},
    }

    _attribute_map = {
        'resource_id': {'key': 'resourceId', 'type': 'str'},
        'load_trust_store': {'key': 'loadTrustStore', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        resource_id: str,
        load_trust_store: Optional[bool] = False,
        **kwargs
    ):
        """
        :keyword resource_id: Required. Resource Id of loaded certificate.
        :paramtype resource_id: str
        :keyword load_trust_store: Indicate whether the certificate will be loaded into default trust
         store, only work for Java runtime.
        :paramtype load_trust_store: bool
        """
        super(LoadedCertificate, self).__init__(**kwargs)
        self.resource_id = resource_id
        self.load_trust_store = load_trust_store


class LogFileUrlResponse(msrest.serialization.Model):
    """Log file URL payload.

    All required parameters must be populated in order to send to Azure.

    :ivar url: Required. URL of the log file.
    :vartype url: str
    """

    _validation = {
        'url': {'required': True},
    }

    _attribute_map = {
        'url': {'key': 'url', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        url: str,
        **kwargs
    ):
        """
        :keyword url: Required. URL of the log file.
        :paramtype url: str
        """
        super(LogFileUrlResponse, self).__init__(**kwargs)
        self.url = url


class LogSpecification(msrest.serialization.Model):
    """Specifications of the Log for Azure Monitoring.

    :ivar name: Name of the log.
    :vartype name: str
    :ivar display_name: Localized friendly display name of the log.
    :vartype display_name: str
    :ivar blob_duration: Blob duration of the log.
    :vartype blob_duration: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'blob_duration': {'key': 'blobDuration', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        display_name: Optional[str] = None,
        blob_duration: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword name: Name of the log.
        :paramtype name: str
        :keyword display_name: Localized friendly display name of the log.
        :paramtype display_name: str
        :keyword blob_duration: Blob duration of the log.
        :paramtype blob_duration: str
        """
        super(LogSpecification, self).__init__(**kwargs)
        self.name = name
        self.display_name = display_name
        self.blob_duration = blob_duration


class ManagedIdentityProperties(msrest.serialization.Model):
    """Managed identity properties retrieved from ARM request headers.

    :ivar type: Type of the managed identity. Possible values include: "None", "SystemAssigned",
     "UserAssigned", "SystemAssigned,UserAssigned".
    :vartype type: str or ~azure.mgmt.appplatform.v2022_04_01.models.ManagedIdentityType
    :ivar principal_id: Principal Id of system-assigned managed identity.
    :vartype principal_id: str
    :ivar tenant_id: Tenant Id of system-assigned managed identity.
    :vartype tenant_id: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'principal_id': {'key': 'principalId', 'type': 'str'},
        'tenant_id': {'key': 'tenantId', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        type: Optional[Union[str, "ManagedIdentityType"]] = None,
        principal_id: Optional[str] = None,
        tenant_id: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword type: Type of the managed identity. Possible values include: "None", "SystemAssigned",
         "UserAssigned", "SystemAssigned,UserAssigned".
        :paramtype type: str or ~azure.mgmt.appplatform.v2022_04_01.models.ManagedIdentityType
        :keyword principal_id: Principal Id of system-assigned managed identity.
        :paramtype principal_id: str
        :keyword tenant_id: Tenant Id of system-assigned managed identity.
        :paramtype tenant_id: str
        """
        super(ManagedIdentityProperties, self).__init__(**kwargs)
        self.type = type
        self.principal_id = principal_id
        self.tenant_id = tenant_id


class MetricDimension(msrest.serialization.Model):
    """Specifications of the Dimension of metrics.

    :ivar name: Name of the dimension.
    :vartype name: str
    :ivar display_name: Localized friendly display name of the dimension.
    :vartype display_name: str
    :ivar to_be_exported_for_shoebox: Whether this dimension should be included for the Shoebox
     export scenario.
    :vartype to_be_exported_for_shoebox: bool
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'to_be_exported_for_shoebox': {'key': 'toBeExportedForShoebox', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        display_name: Optional[str] = None,
        to_be_exported_for_shoebox: Optional[bool] = None,
        **kwargs
    ):
        """
        :keyword name: Name of the dimension.
        :paramtype name: str
        :keyword display_name: Localized friendly display name of the dimension.
        :paramtype display_name: str
        :keyword to_be_exported_for_shoebox: Whether this dimension should be included for the Shoebox
         export scenario.
        :paramtype to_be_exported_for_shoebox: bool
        """
        super(MetricDimension, self).__init__(**kwargs)
        self.name = name
        self.display_name = display_name
        self.to_be_exported_for_shoebox = to_be_exported_for_shoebox


class MetricSpecification(msrest.serialization.Model):
    """Specifications of the Metrics for Azure Monitoring.

    :ivar name: Name of the metric.
    :vartype name: str
    :ivar display_name: Localized friendly display name of the metric.
    :vartype display_name: str
    :ivar display_description: Localized friendly description of the metric.
    :vartype display_description: str
    :ivar unit: Unit that makes sense for the metric.
    :vartype unit: str
    :ivar category: Name of the metric category that the metric belongs to. A metric can only
     belong to a single category.
    :vartype category: str
    :ivar aggregation_type: Only provide one value for this field. Valid values: Average, Minimum,
     Maximum, Total, Count.
    :vartype aggregation_type: str
    :ivar supported_aggregation_types: Supported aggregation types.
    :vartype supported_aggregation_types: list[str]
    :ivar supported_time_grain_types: Supported time grain types.
    :vartype supported_time_grain_types: list[str]
    :ivar fill_gap_with_zero: Optional. If set to true, then zero will be returned for time
     duration where no metric is emitted/published.
    :vartype fill_gap_with_zero: bool
    :ivar dimensions: Dimensions of the metric.
    :vartype dimensions: list[~azure.mgmt.appplatform.v2022_04_01.models.MetricDimension]
    :ivar source_mdm_namespace: Name of the MDM namespace. Optional.
    :vartype source_mdm_namespace: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'display_description': {'key': 'displayDescription', 'type': 'str'},
        'unit': {'key': 'unit', 'type': 'str'},
        'category': {'key': 'category', 'type': 'str'},
        'aggregation_type': {'key': 'aggregationType', 'type': 'str'},
        'supported_aggregation_types': {'key': 'supportedAggregationTypes', 'type': '[str]'},
        'supported_time_grain_types': {'key': 'supportedTimeGrainTypes', 'type': '[str]'},
        'fill_gap_with_zero': {'key': 'fillGapWithZero', 'type': 'bool'},
        'dimensions': {'key': 'dimensions', 'type': '[MetricDimension]'},
        'source_mdm_namespace': {'key': 'sourceMdmNamespace', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        display_name: Optional[str] = None,
        display_description: Optional[str] = None,
        unit: Optional[str] = None,
        category: Optional[str] = None,
        aggregation_type: Optional[str] = None,
        supported_aggregation_types: Optional[List[str]] = None,
        supported_time_grain_types: Optional[List[str]] = None,
        fill_gap_with_zero: Optional[bool] = None,
        dimensions: Optional[List["MetricDimension"]] = None,
        source_mdm_namespace: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword name: Name of the metric.
        :paramtype name: str
        :keyword display_name: Localized friendly display name of the metric.
        :paramtype display_name: str
        :keyword display_description: Localized friendly description of the metric.
        :paramtype display_description: str
        :keyword unit: Unit that makes sense for the metric.
        :paramtype unit: str
        :keyword category: Name of the metric category that the metric belongs to. A metric can only
         belong to a single category.
        :paramtype category: str
        :keyword aggregation_type: Only provide one value for this field. Valid values: Average,
         Minimum, Maximum, Total, Count.
        :paramtype aggregation_type: str
        :keyword supported_aggregation_types: Supported aggregation types.
        :paramtype supported_aggregation_types: list[str]
        :keyword supported_time_grain_types: Supported time grain types.
        :paramtype supported_time_grain_types: list[str]
        :keyword fill_gap_with_zero: Optional. If set to true, then zero will be returned for time
         duration where no metric is emitted/published.
        :paramtype fill_gap_with_zero: bool
        :keyword dimensions: Dimensions of the metric.
        :paramtype dimensions: list[~azure.mgmt.appplatform.v2022_04_01.models.MetricDimension]
        :keyword source_mdm_namespace: Name of the MDM namespace. Optional.
        :paramtype source_mdm_namespace: str
        """
        super(MetricSpecification, self).__init__(**kwargs)
        self.name = name
        self.display_name = display_name
        self.display_description = display_description
        self.unit = unit
        self.category = category
        self.aggregation_type = aggregation_type
        self.supported_aggregation_types = supported_aggregation_types
        self.supported_time_grain_types = supported_time_grain_types
        self.fill_gap_with_zero = fill_gap_with_zero
        self.dimensions = dimensions
        self.source_mdm_namespace = source_mdm_namespace


class MonitoringSettingProperties(msrest.serialization.Model):
    """Monitoring Setting properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar provisioning_state: State of the Monitoring Setting. Possible values include:
     "NotAvailable", "Failed", "Succeeded", "Updating".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.MonitoringSettingState
    :ivar error: Error when apply Monitoring Setting changes.
    :vartype error: ~azure.mgmt.appplatform.v2022_04_01.models.Error
    :ivar trace_enabled: Indicates whether enable the trace functionality, which will be deprecated
     since api version 2020-11-01-preview. Please leverage appInsightsInstrumentationKey to indicate
     if monitoringSettings enabled or not.
    :vartype trace_enabled: bool
    :ivar app_insights_instrumentation_key: Target application insight instrumentation key, null or
     whitespace include empty will disable monitoringSettings.
    :vartype app_insights_instrumentation_key: str
    :ivar app_insights_sampling_rate: Indicates the sampling rate of application insight agent,
     should be in range [0.0, 100.0].
    :vartype app_insights_sampling_rate: float
    :ivar app_insights_agent_versions: Indicates the versions of application insight agent.
    :vartype app_insights_agent_versions:
     ~azure.mgmt.appplatform.v2022_04_01.models.ApplicationInsightsAgentVersions
    """

    _validation = {
        'provisioning_state': {'readonly': True},
        'app_insights_sampling_rate': {'maximum': 100, 'minimum': 0},
    }

    _attribute_map = {
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'error': {'key': 'error', 'type': 'Error'},
        'trace_enabled': {'key': 'traceEnabled', 'type': 'bool'},
        'app_insights_instrumentation_key': {'key': 'appInsightsInstrumentationKey', 'type': 'str'},
        'app_insights_sampling_rate': {'key': 'appInsightsSamplingRate', 'type': 'float'},
        'app_insights_agent_versions': {'key': 'appInsightsAgentVersions', 'type': 'ApplicationInsightsAgentVersions'},
    }

    def __init__(
        self,
        *,
        error: Optional["Error"] = None,
        trace_enabled: Optional[bool] = None,
        app_insights_instrumentation_key: Optional[str] = None,
        app_insights_sampling_rate: Optional[float] = None,
        app_insights_agent_versions: Optional["ApplicationInsightsAgentVersions"] = None,
        **kwargs
    ):
        """
        :keyword error: Error when apply Monitoring Setting changes.
        :paramtype error: ~azure.mgmt.appplatform.v2022_04_01.models.Error
        :keyword trace_enabled: Indicates whether enable the trace functionality, which will be
         deprecated since api version 2020-11-01-preview. Please leverage appInsightsInstrumentationKey
         to indicate if monitoringSettings enabled or not.
        :paramtype trace_enabled: bool
        :keyword app_insights_instrumentation_key: Target application insight instrumentation key, null
         or whitespace include empty will disable monitoringSettings.
        :paramtype app_insights_instrumentation_key: str
        :keyword app_insights_sampling_rate: Indicates the sampling rate of application insight agent,
         should be in range [0.0, 100.0].
        :paramtype app_insights_sampling_rate: float
        :keyword app_insights_agent_versions: Indicates the versions of application insight agent.
        :paramtype app_insights_agent_versions:
         ~azure.mgmt.appplatform.v2022_04_01.models.ApplicationInsightsAgentVersions
        """
        super(MonitoringSettingProperties, self).__init__(**kwargs)
        self.provisioning_state = None
        self.error = error
        self.trace_enabled = trace_enabled
        self.app_insights_instrumentation_key = app_insights_instrumentation_key
        self.app_insights_sampling_rate = app_insights_sampling_rate
        self.app_insights_agent_versions = app_insights_agent_versions


class MonitoringSettingResource(ProxyResource):
    """Monitoring Setting resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Properties of the Monitoring Setting resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.MonitoringSettingProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'MonitoringSettingProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["MonitoringSettingProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Properties of the Monitoring Setting resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.MonitoringSettingProperties
        """
        super(MonitoringSettingResource, self).__init__(**kwargs)
        self.properties = properties


class NameAvailability(msrest.serialization.Model):
    """Name availability result payload.

    :ivar name_available: Indicates whether the name is available.
    :vartype name_available: bool
    :ivar reason: Reason why the name is not available.
    :vartype reason: str
    :ivar message: Message why the name is not available.
    :vartype message: str
    """

    _attribute_map = {
        'name_available': {'key': 'nameAvailable', 'type': 'bool'},
        'reason': {'key': 'reason', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name_available: Optional[bool] = None,
        reason: Optional[str] = None,
        message: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword name_available: Indicates whether the name is available.
        :paramtype name_available: bool
        :keyword reason: Reason why the name is not available.
        :paramtype reason: str
        :keyword message: Message why the name is not available.
        :paramtype message: str
        """
        super(NameAvailability, self).__init__(**kwargs)
        self.name_available = name_available
        self.reason = reason
        self.message = message


class NameAvailabilityParameters(msrest.serialization.Model):
    """Name availability parameters payload.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of the resource to check name availability.
    :vartype type: str
    :ivar name: Required. Name to be checked.
    :vartype name: str
    """

    _validation = {
        'type': {'required': True},
        'name': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        type: str,
        name: str,
        **kwargs
    ):
        """
        :keyword type: Required. Type of the resource to check name availability.
        :paramtype type: str
        :keyword name: Required. Name to be checked.
        :paramtype name: str
        """
        super(NameAvailabilityParameters, self).__init__(**kwargs)
        self.type = type
        self.name = name


class NetCoreZipUploadedUserSourceInfo(UploadedUserSourceInfo):
    """Uploaded Jar binary for a deployment.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of the source uploaded.Constant filled by server.
    :vartype type: str
    :ivar version: Version of the source.
    :vartype version: str
    :ivar relative_path: Relative path of the storage which stores the source.
    :vartype relative_path: str
    :ivar net_core_main_entry_path: The path to the .NET executable relative to zip root.
    :vartype net_core_main_entry_path: str
    :ivar runtime_version: Runtime version of the .Net file.
    :vartype runtime_version: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'relative_path': {'key': 'relativePath', 'type': 'str'},
        'net_core_main_entry_path': {'key': 'netCoreMainEntryPath', 'type': 'str'},
        'runtime_version': {'key': 'runtimeVersion', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        version: Optional[str] = None,
        relative_path: Optional[str] = None,
        net_core_main_entry_path: Optional[str] = None,
        runtime_version: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword version: Version of the source.
        :paramtype version: str
        :keyword relative_path: Relative path of the storage which stores the source.
        :paramtype relative_path: str
        :keyword net_core_main_entry_path: The path to the .NET executable relative to zip root.
        :paramtype net_core_main_entry_path: str
        :keyword runtime_version: Runtime version of the .Net file.
        :paramtype runtime_version: str
        """
        super(NetCoreZipUploadedUserSourceInfo, self).__init__(version=version, relative_path=relative_path, **kwargs)
        self.type = 'NetCoreZip'  # type: str
        self.net_core_main_entry_path = net_core_main_entry_path
        self.runtime_version = runtime_version


class NetworkProfile(msrest.serialization.Model):
    """Service network profile payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar service_runtime_subnet_id: Fully qualified resource Id of the subnet to host Azure Spring
     Apps Service Runtime.
    :vartype service_runtime_subnet_id: str
    :ivar app_subnet_id: Fully qualified resource Id of the subnet to host customer apps in Azure
     Spring Apps.
    :vartype app_subnet_id: str
    :ivar service_cidr: Azure Spring Apps service reserved CIDR.
    :vartype service_cidr: str
    :ivar service_runtime_network_resource_group: Name of the resource group containing network
     resources of Azure Spring Apps Service Runtime.
    :vartype service_runtime_network_resource_group: str
    :ivar app_network_resource_group: Name of the resource group containing network resources for
     customer apps in Azure Spring Apps.
    :vartype app_network_resource_group: str
    :ivar outbound_i_ps: Desired outbound IP resources for Azure Spring Apps resource.
    :vartype outbound_i_ps: ~azure.mgmt.appplatform.v2022_04_01.models.NetworkProfileOutboundIPs
    :ivar required_traffics: Required inbound or outbound traffics for Azure Spring Apps resource.
    :vartype required_traffics: list[~azure.mgmt.appplatform.v2022_04_01.models.RequiredTraffic]
    """

    _validation = {
        'outbound_i_ps': {'readonly': True},
        'required_traffics': {'readonly': True},
    }

    _attribute_map = {
        'service_runtime_subnet_id': {'key': 'serviceRuntimeSubnetId', 'type': 'str'},
        'app_subnet_id': {'key': 'appSubnetId', 'type': 'str'},
        'service_cidr': {'key': 'serviceCidr', 'type': 'str'},
        'service_runtime_network_resource_group': {'key': 'serviceRuntimeNetworkResourceGroup', 'type': 'str'},
        'app_network_resource_group': {'key': 'appNetworkResourceGroup', 'type': 'str'},
        'outbound_i_ps': {'key': 'outboundIPs', 'type': 'NetworkProfileOutboundIPs'},
        'required_traffics': {'key': 'requiredTraffics', 'type': '[RequiredTraffic]'},
    }

    def __init__(
        self,
        *,
        service_runtime_subnet_id: Optional[str] = None,
        app_subnet_id: Optional[str] = None,
        service_cidr: Optional[str] = None,
        service_runtime_network_resource_group: Optional[str] = None,
        app_network_resource_group: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword service_runtime_subnet_id: Fully qualified resource Id of the subnet to host Azure
         Spring Apps Service Runtime.
        :paramtype service_runtime_subnet_id: str
        :keyword app_subnet_id: Fully qualified resource Id of the subnet to host customer apps in
         Azure Spring Apps.
        :paramtype app_subnet_id: str
        :keyword service_cidr: Azure Spring Apps service reserved CIDR.
        :paramtype service_cidr: str
        :keyword service_runtime_network_resource_group: Name of the resource group containing network
         resources of Azure Spring Apps Service Runtime.
        :paramtype service_runtime_network_resource_group: str
        :keyword app_network_resource_group: Name of the resource group containing network resources
         for customer apps in Azure Spring Apps.
        :paramtype app_network_resource_group: str
        """
        super(NetworkProfile, self).__init__(**kwargs)
        self.service_runtime_subnet_id = service_runtime_subnet_id
        self.app_subnet_id = app_subnet_id
        self.service_cidr = service_cidr
        self.service_runtime_network_resource_group = service_runtime_network_resource_group
        self.app_network_resource_group = app_network_resource_group
        self.outbound_i_ps = None
        self.required_traffics = None


class NetworkProfileOutboundIPs(msrest.serialization.Model):
    """Desired outbound IP resources for Azure Spring Apps resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar public_i_ps: A list of public IP addresses.
    :vartype public_i_ps: list[str]
    """

    _validation = {
        'public_i_ps': {'readonly': True},
    }

    _attribute_map = {
        'public_i_ps': {'key': 'publicIPs', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(NetworkProfileOutboundIPs, self).__init__(**kwargs)
        self.public_i_ps = None


class OperationDetail(msrest.serialization.Model):
    """Operation detail payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: Name of the operation.
    :vartype name: str
    :ivar is_data_action: Indicates whether the operation is a data action.
    :vartype is_data_action: bool
    :ivar display: Display of the operation.
    :vartype display: ~azure.mgmt.appplatform.v2022_04_01.models.OperationDisplay
    :ivar action_type: Enum. Indicates the action type. "Internal" refers to actions that are for
     internal only APIs. Possible values include: "Internal".
    :vartype action_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.ActionType
    :ivar origin: Origin of the operation.
    :vartype origin: str
    :ivar properties: Properties of the operation.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.OperationProperties
    """

    _validation = {
        'action_type': {'readonly': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'is_data_action': {'key': 'isDataAction', 'type': 'bool'},
        'display': {'key': 'display', 'type': 'OperationDisplay'},
        'action_type': {'key': 'actionType', 'type': 'str'},
        'origin': {'key': 'origin', 'type': 'str'},
        'properties': {'key': 'properties', 'type': 'OperationProperties'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        is_data_action: Optional[bool] = None,
        display: Optional["OperationDisplay"] = None,
        origin: Optional[str] = None,
        properties: Optional["OperationProperties"] = None,
        **kwargs
    ):
        """
        :keyword name: Name of the operation.
        :paramtype name: str
        :keyword is_data_action: Indicates whether the operation is a data action.
        :paramtype is_data_action: bool
        :keyword display: Display of the operation.
        :paramtype display: ~azure.mgmt.appplatform.v2022_04_01.models.OperationDisplay
        :keyword origin: Origin of the operation.
        :paramtype origin: str
        :keyword properties: Properties of the operation.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.OperationProperties
        """
        super(OperationDetail, self).__init__(**kwargs)
        self.name = name
        self.is_data_action = is_data_action
        self.display = display
        self.action_type = None
        self.origin = origin
        self.properties = properties


class OperationDisplay(msrest.serialization.Model):
    """Operation display payload.

    :ivar provider: Resource provider of the operation.
    :vartype provider: str
    :ivar resource: Resource of the operation.
    :vartype resource: str
    :ivar operation: Localized friendly name for the operation.
    :vartype operation: str
    :ivar description: Localized friendly description for the operation.
    :vartype description: str
    """

    _attribute_map = {
        'provider': {'key': 'provider', 'type': 'str'},
        'resource': {'key': 'resource', 'type': 'str'},
        'operation': {'key': 'operation', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        provider: Optional[str] = None,
        resource: Optional[str] = None,
        operation: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword provider: Resource provider of the operation.
        :paramtype provider: str
        :keyword resource: Resource of the operation.
        :paramtype resource: str
        :keyword operation: Localized friendly name for the operation.
        :paramtype operation: str
        :keyword description: Localized friendly description for the operation.
        :paramtype description: str
        """
        super(OperationDisplay, self).__init__(**kwargs)
        self.provider = provider
        self.resource = resource
        self.operation = operation
        self.description = description


class OperationProperties(msrest.serialization.Model):
    """Extra Operation properties.

    :ivar service_specification: Service specifications of the operation.
    :vartype service_specification: ~azure.mgmt.appplatform.v2022_04_01.models.ServiceSpecification
    """

    _attribute_map = {
        'service_specification': {'key': 'serviceSpecification', 'type': 'ServiceSpecification'},
    }

    def __init__(
        self,
        *,
        service_specification: Optional["ServiceSpecification"] = None,
        **kwargs
    ):
        """
        :keyword service_specification: Service specifications of the operation.
        :paramtype service_specification:
         ~azure.mgmt.appplatform.v2022_04_01.models.ServiceSpecification
        """
        super(OperationProperties, self).__init__(**kwargs)
        self.service_specification = service_specification


class PersistentDisk(msrest.serialization.Model):
    """Persistent disk payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar size_in_gb: Size of the persistent disk in GB.
    :vartype size_in_gb: int
    :ivar used_in_gb: Size of the used persistent disk in GB.
    :vartype used_in_gb: int
    :ivar mount_path: Mount path of the persistent disk.
    :vartype mount_path: str
    """

    _validation = {
        'size_in_gb': {'maximum': 50, 'minimum': 0},
        'used_in_gb': {'readonly': True, 'maximum': 50, 'minimum': 0},
    }

    _attribute_map = {
        'size_in_gb': {'key': 'sizeInGB', 'type': 'int'},
        'used_in_gb': {'key': 'usedInGB', 'type': 'int'},
        'mount_path': {'key': 'mountPath', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        size_in_gb: Optional[int] = None,
        mount_path: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword size_in_gb: Size of the persistent disk in GB.
        :paramtype size_in_gb: int
        :keyword mount_path: Mount path of the persistent disk.
        :paramtype mount_path: str
        """
        super(PersistentDisk, self).__init__(**kwargs)
        self.size_in_gb = size_in_gb
        self.used_in_gb = None
        self.mount_path = mount_path


class RegenerateTestKeyRequestPayload(msrest.serialization.Model):
    """Regenerate test key request payload.

    All required parameters must be populated in order to send to Azure.

    :ivar key_type: Required. Type of the test key. Possible values include: "Primary",
     "Secondary".
    :vartype key_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.TestKeyType
    """

    _validation = {
        'key_type': {'required': True},
    }

    _attribute_map = {
        'key_type': {'key': 'keyType', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        key_type: Union[str, "TestKeyType"],
        **kwargs
    ):
        """
        :keyword key_type: Required. Type of the test key. Possible values include: "Primary",
         "Secondary".
        :paramtype key_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.TestKeyType
        """
        super(RegenerateTestKeyRequestPayload, self).__init__(**kwargs)
        self.key_type = key_type


class RequiredTraffic(msrest.serialization.Model):
    """Required inbound or outbound traffic for Azure Spring Apps resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar protocol: The protocol of required traffic.
    :vartype protocol: str
    :ivar port: The port of required traffic.
    :vartype port: int
    :ivar ips: The ip list of required traffic.
    :vartype ips: list[str]
    :ivar fqdns: The FQDN list of required traffic.
    :vartype fqdns: list[str]
    :ivar direction: The direction of required traffic. Possible values include: "Inbound",
     "Outbound".
    :vartype direction: str or ~azure.mgmt.appplatform.v2022_04_01.models.TrafficDirection
    """

    _validation = {
        'protocol': {'readonly': True},
        'port': {'readonly': True},
        'ips': {'readonly': True},
        'fqdns': {'readonly': True},
        'direction': {'readonly': True},
    }

    _attribute_map = {
        'protocol': {'key': 'protocol', 'type': 'str'},
        'port': {'key': 'port', 'type': 'int'},
        'ips': {'key': 'ips', 'type': '[str]'},
        'fqdns': {'key': 'fqdns', 'type': '[str]'},
        'direction': {'key': 'direction', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(RequiredTraffic, self).__init__(**kwargs)
        self.protocol = None
        self.port = None
        self.ips = None
        self.fqdns = None
        self.direction = None


class ResourceRequests(msrest.serialization.Model):
    """Deployment resource request payload.

    :ivar cpu: Required CPU. 1 core can be represented by 1 or 1000m. This should be 500m or 1 for
     Basic tier, and {500m, 1, 2, 3, 4} for Standard tier.
    :vartype cpu: str
    :ivar memory: Required memory. 1 GB can be represented by 1Gi or 1024Mi. This should be {512Mi,
     1Gi, 2Gi} for Basic tier, and {512Mi, 1Gi, 2Gi, ..., 8Gi} for Standard tier.
    :vartype memory: str
    """

    _attribute_map = {
        'cpu': {'key': 'cpu', 'type': 'str'},
        'memory': {'key': 'memory', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        cpu: Optional[str] = None,
        memory: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword cpu: Required CPU. 1 core can be represented by 1 or 1000m. This should be 500m or 1
         for Basic tier, and {500m, 1, 2, 3, 4} for Standard tier.
        :paramtype cpu: str
        :keyword memory: Required memory. 1 GB can be represented by 1Gi or 1024Mi. This should be
         {512Mi, 1Gi, 2Gi} for Basic tier, and {512Mi, 1Gi, 2Gi, ..., 8Gi} for Standard tier.
        :paramtype memory: str
        """
        super(ResourceRequests, self).__init__(**kwargs)
        self.cpu = cpu
        self.memory = memory


class ResourceSku(msrest.serialization.Model):
    """Describes an available Azure Spring Apps SKU.

    :ivar resource_type: Gets the type of resource the SKU applies to.
    :vartype resource_type: str
    :ivar name: Gets the name of SKU.
    :vartype name: str
    :ivar tier: Gets the tier of SKU.
    :vartype tier: str
    :ivar capacity: Gets the capacity of SKU.
    :vartype capacity: ~azure.mgmt.appplatform.v2022_04_01.models.SkuCapacity
    :ivar locations: Gets the set of locations that the SKU is available.
    :vartype locations: list[str]
    :ivar location_info: Gets a list of locations and availability zones in those locations where
     the SKU is available.
    :vartype location_info:
     list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuLocationInfo]
    :ivar restrictions: Gets the restrictions because of which SKU cannot be used. This is
     empty if there are no restrictions.
    :vartype restrictions: list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictions]
    """

    _attribute_map = {
        'resource_type': {'key': 'resourceType', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'tier': {'key': 'tier', 'type': 'str'},
        'capacity': {'key': 'capacity', 'type': 'SkuCapacity'},
        'locations': {'key': 'locations', 'type': '[str]'},
        'location_info': {'key': 'locationInfo', 'type': '[ResourceSkuLocationInfo]'},
        'restrictions': {'key': 'restrictions', 'type': '[ResourceSkuRestrictions]'},
    }

    def __init__(
        self,
        *,
        resource_type: Optional[str] = None,
        name: Optional[str] = None,
        tier: Optional[str] = None,
        capacity: Optional["SkuCapacity"] = None,
        locations: Optional[List[str]] = None,
        location_info: Optional[List["ResourceSkuLocationInfo"]] = None,
        restrictions: Optional[List["ResourceSkuRestrictions"]] = None,
        **kwargs
    ):
        """
        :keyword resource_type: Gets the type of resource the SKU applies to.
        :paramtype resource_type: str
        :keyword name: Gets the name of SKU.
        :paramtype name: str
        :keyword tier: Gets the tier of SKU.
        :paramtype tier: str
        :keyword capacity: Gets the capacity of SKU.
        :paramtype capacity: ~azure.mgmt.appplatform.v2022_04_01.models.SkuCapacity
        :keyword locations: Gets the set of locations that the SKU is available.
        :paramtype locations: list[str]
        :keyword location_info: Gets a list of locations and availability zones in those locations
         where the SKU is available.
        :paramtype location_info:
         list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuLocationInfo]
        :keyword restrictions: Gets the restrictions because of which SKU cannot be used. This is
         empty if there are no restrictions.
        :paramtype restrictions:
         list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictions]
        """
        super(ResourceSku, self).__init__(**kwargs)
        self.resource_type = resource_type
        self.name = name
        self.tier = tier
        self.capacity = capacity
        self.locations = locations
        self.location_info = location_info
        self.restrictions = restrictions


class ResourceSkuCapabilities(msrest.serialization.Model):
    """ResourceSkuCapabilities.

    :ivar name: Gets an invariant to describe the feature.
    :vartype name: str
    :ivar value: Gets an invariant if the feature is measured by quantity.
    :vartype value: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        value: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword name: Gets an invariant to describe the feature.
        :paramtype name: str
        :keyword value: Gets an invariant if the feature is measured by quantity.
        :paramtype value: str
        """
        super(ResourceSkuCapabilities, self).__init__(**kwargs)
        self.name = name
        self.value = value


class ResourceSkuCollection(msrest.serialization.Model):
    """Object that includes an array of Azure Spring Apps SKU and a possible link for next set.

    :ivar value: Collection of resource SKU.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSku]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[ResourceSku]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["ResourceSku"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of resource SKU.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSku]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(ResourceSkuCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ResourceSkuLocationInfo(msrest.serialization.Model):
    """Locations and availability zones where the SKU is available.

    :ivar location: Gets location of the SKU.
    :vartype location: str
    :ivar zones: Gets list of availability zones where the SKU is supported.
    :vartype zones: list[str]
    :ivar zone_details: Gets details of capabilities available to a SKU in specific zones.
    :vartype zone_details: list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuZoneDetails]
    """

    _attribute_map = {
        'location': {'key': 'location', 'type': 'str'},
        'zones': {'key': 'zones', 'type': '[str]'},
        'zone_details': {'key': 'zoneDetails', 'type': '[ResourceSkuZoneDetails]'},
    }

    def __init__(
        self,
        *,
        location: Optional[str] = None,
        zones: Optional[List[str]] = None,
        zone_details: Optional[List["ResourceSkuZoneDetails"]] = None,
        **kwargs
    ):
        """
        :keyword location: Gets location of the SKU.
        :paramtype location: str
        :keyword zones: Gets list of availability zones where the SKU is supported.
        :paramtype zones: list[str]
        :keyword zone_details: Gets details of capabilities available to a SKU in specific zones.
        :paramtype zone_details:
         list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuZoneDetails]
        """
        super(ResourceSkuLocationInfo, self).__init__(**kwargs)
        self.location = location
        self.zones = zones
        self.zone_details = zone_details


class ResourceSkuRestrictionInfo(msrest.serialization.Model):
    """Information about the restriction where the SKU cannot be used.

    :ivar locations: Gets locations where the SKU is restricted.
    :vartype locations: list[str]
    :ivar zones: Gets list of availability zones where the SKU is restricted.
    :vartype zones: list[str]
    """

    _attribute_map = {
        'locations': {'key': 'locations', 'type': '[str]'},
        'zones': {'key': 'zones', 'type': '[str]'},
    }

    def __init__(
        self,
        *,
        locations: Optional[List[str]] = None,
        zones: Optional[List[str]] = None,
        **kwargs
    ):
        """
        :keyword locations: Gets locations where the SKU is restricted.
        :paramtype locations: list[str]
        :keyword zones: Gets list of availability zones where the SKU is restricted.
        :paramtype zones: list[str]
        """
        super(ResourceSkuRestrictionInfo, self).__init__(**kwargs)
        self.locations = locations
        self.zones = zones


class ResourceSkuRestrictions(msrest.serialization.Model):
    """Restrictions where the SKU cannot be used.

    :ivar type: Gets the type of restrictions. Possible values include: 'Location', 'Zone'.
     Possible values include: "Location", "Zone".
    :vartype type: str or ~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictionsType
    :ivar values: Gets the value of restrictions. If the restriction type is set to
     location. This would be different locations where the SKU is restricted.
    :vartype values: list[str]
    :ivar restriction_info: Gets the information about the restriction where the SKU cannot be
     used.
    :vartype restriction_info:
     ~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictionInfo
    :ivar reason_code: Gets the reason for restriction. Possible values include: 'QuotaId',
     'NotAvailableForSubscription'. Possible values include: "QuotaId",
     "NotAvailableForSubscription".
    :vartype reason_code: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictionsReasonCode
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'values': {'key': 'values', 'type': '[str]'},
        'restriction_info': {'key': 'restrictionInfo', 'type': 'ResourceSkuRestrictionInfo'},
        'reason_code': {'key': 'reasonCode', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        type: Optional[Union[str, "ResourceSkuRestrictionsType"]] = None,
        values: Optional[List[str]] = None,
        restriction_info: Optional["ResourceSkuRestrictionInfo"] = None,
        reason_code: Optional[Union[str, "ResourceSkuRestrictionsReasonCode"]] = None,
        **kwargs
    ):
        """
        :keyword type: Gets the type of restrictions. Possible values include: 'Location', 'Zone'.
         Possible values include: "Location", "Zone".
        :paramtype type: str or ~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictionsType
        :keyword values: Gets the value of restrictions. If the restriction type is set to
         location. This would be different locations where the SKU is restricted.
        :paramtype values: list[str]
        :keyword restriction_info: Gets the information about the restriction where the SKU cannot be
         used.
        :paramtype restriction_info:
         ~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictionInfo
        :keyword reason_code: Gets the reason for restriction. Possible values include: 'QuotaId',
         'NotAvailableForSubscription'. Possible values include: "QuotaId",
         "NotAvailableForSubscription".
        :paramtype reason_code: str or
         ~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuRestrictionsReasonCode
        """
        super(ResourceSkuRestrictions, self).__init__(**kwargs)
        self.type = type
        self.values = values
        self.restriction_info = restriction_info
        self.reason_code = reason_code


class ResourceSkuZoneDetails(msrest.serialization.Model):
    """Details of capabilities available to a SKU in specific zones.

    :ivar name: Gets the set of zones that the SKU is available in with the
     specified capabilities.
    :vartype name: list[str]
    :ivar capabilities: Gets a list of capabilities that are available for the SKU in the
     specified list of zones.
    :vartype capabilities: list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuCapabilities]
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': '[str]'},
        'capabilities': {'key': 'capabilities', 'type': '[ResourceSkuCapabilities]'},
    }

    def __init__(
        self,
        *,
        name: Optional[List[str]] = None,
        capabilities: Optional[List["ResourceSkuCapabilities"]] = None,
        **kwargs
    ):
        """
        :keyword name: Gets the set of zones that the SKU is available in with the
         specified capabilities.
        :paramtype name: list[str]
        :keyword capabilities: Gets a list of capabilities that are available for the SKU in the
         specified list of zones.
        :paramtype capabilities:
         list[~azure.mgmt.appplatform.v2022_04_01.models.ResourceSkuCapabilities]
        """
        super(ResourceSkuZoneDetails, self).__init__(**kwargs)
        self.name = name
        self.capabilities = capabilities


class ResourceUploadDefinition(msrest.serialization.Model):
    """Resource upload definition payload.

    :ivar relative_path: Source relative path.
    :vartype relative_path: str
    :ivar upload_url: Upload URL.
    :vartype upload_url: str
    """

    _attribute_map = {
        'relative_path': {'key': 'relativePath', 'type': 'str'},
        'upload_url': {'key': 'uploadUrl', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        relative_path: Optional[str] = None,
        upload_url: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword relative_path: Source relative path.
        :paramtype relative_path: str
        :keyword upload_url: Upload URL.
        :paramtype upload_url: str
        """
        super(ResourceUploadDefinition, self).__init__(**kwargs)
        self.relative_path = relative_path
        self.upload_url = upload_url


class ServiceRegistryInstance(msrest.serialization.Model):
    """Collection of instances belong to the Service Registry.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: Name of the Service Registry instance.
    :vartype name: str
    :ivar status: Status of the Service Registry instance.
    :vartype status: str
    """

    _validation = {
        'name': {'readonly': True},
        'status': {'readonly': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(ServiceRegistryInstance, self).__init__(**kwargs)
        self.name = None
        self.status = None


class ServiceRegistryProperties(msrest.serialization.Model):
    """Service Registry properties payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar provisioning_state: State of the Service Registry. Possible values include: "Creating",
     "Updating", "Succeeded", "Failed", "Deleting".
    :vartype provisioning_state: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.ServiceRegistryProvisioningState
    :ivar resource_requests: The requested resource quantity for required CPU and Memory.
    :vartype resource_requests:
     ~azure.mgmt.appplatform.v2022_04_01.models.ServiceRegistryResourceRequests
    :ivar instances: Collection of instances belong to Service Registry.
    :vartype instances: list[~azure.mgmt.appplatform.v2022_04_01.models.ServiceRegistryInstance]
    """

    _validation = {
        'provisioning_state': {'readonly': True},
        'resource_requests': {'readonly': True},
        'instances': {'readonly': True},
    }

    _attribute_map = {
        'provisioning_state': {'key': 'provisioningState', 'type': 'str'},
        'resource_requests': {'key': 'resourceRequests', 'type': 'ServiceRegistryResourceRequests'},
        'instances': {'key': 'instances', 'type': '[ServiceRegistryInstance]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(ServiceRegistryProperties, self).__init__(**kwargs)
        self.provisioning_state = None
        self.resource_requests = None
        self.instances = None


class ServiceRegistryResource(ProxyResource):
    """Service Registry resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Service Registry properties payload.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.ServiceRegistryProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'ServiceRegistryProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["ServiceRegistryProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Service Registry properties payload.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.ServiceRegistryProperties
        """
        super(ServiceRegistryResource, self).__init__(**kwargs)
        self.properties = properties


class ServiceRegistryResourceCollection(msrest.serialization.Model):
    """Object that includes an array of Service Registry resources and a possible link for next set.

    :ivar value: Collection of Service Registry resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ServiceRegistryResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[ServiceRegistryResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["ServiceRegistryResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Service Registry resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ServiceRegistryResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(ServiceRegistryResourceCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ServiceRegistryResourceRequests(msrest.serialization.Model):
    """Resource request payload of Service Registry.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar cpu: Cpu allocated to each Service Registry instance.
    :vartype cpu: str
    :ivar memory: Memory allocated to each Service Registry instance.
    :vartype memory: str
    :ivar instance_count: Instance count of the Service Registry.
    :vartype instance_count: int
    """

    _validation = {
        'cpu': {'readonly': True},
        'memory': {'readonly': True},
        'instance_count': {'readonly': True},
    }

    _attribute_map = {
        'cpu': {'key': 'cpu', 'type': 'str'},
        'memory': {'key': 'memory', 'type': 'str'},
        'instance_count': {'key': 'instanceCount', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        """
        """
        super(ServiceRegistryResourceRequests, self).__init__(**kwargs)
        self.cpu = None
        self.memory = None
        self.instance_count = None


class TrackedResource(Resource):
    """The resource model definition for a ARM tracked top level resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar location: The GEO location of the resource.
    :vartype location: str
    :ivar tags: A set of tags. Tags of the service which is a list of key value pairs that describe
     the resource.
    :vartype tags: dict[str, str]
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'location': {'key': 'location', 'type': 'str'},
        'tags': {'key': 'tags', 'type': '{str}'},
    }

    def __init__(
        self,
        *,
        location: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
        **kwargs
    ):
        """
        :keyword location: The GEO location of the resource.
        :paramtype location: str
        :keyword tags: A set of tags. Tags of the service which is a list of key value pairs that
         describe the resource.
        :paramtype tags: dict[str, str]
        """
        super(TrackedResource, self).__init__(**kwargs)
        self.location = location
        self.tags = tags


class ServiceResource(TrackedResource):
    """Service resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar location: The GEO location of the resource.
    :vartype location: str
    :ivar tags: A set of tags. Tags of the service which is a list of key value pairs that describe
     the resource.
    :vartype tags: dict[str, str]
    :ivar properties: Properties of the Service resource.
    :vartype properties: ~azure.mgmt.appplatform.v2022_04_01.models.ClusterResourceProperties
    :ivar sku: Sku of the Service resource.
    :vartype sku: ~azure.mgmt.appplatform.v2022_04_01.models.Sku
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'location': {'key': 'location', 'type': 'str'},
        'tags': {'key': 'tags', 'type': '{str}'},
        'properties': {'key': 'properties', 'type': 'ClusterResourceProperties'},
        'sku': {'key': 'sku', 'type': 'Sku'},
    }

    def __init__(
        self,
        *,
        location: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
        properties: Optional["ClusterResourceProperties"] = None,
        sku: Optional["Sku"] = None,
        **kwargs
    ):
        """
        :keyword location: The GEO location of the resource.
        :paramtype location: str
        :keyword tags: A set of tags. Tags of the service which is a list of key value pairs that
         describe the resource.
        :paramtype tags: dict[str, str]
        :keyword properties: Properties of the Service resource.
        :paramtype properties: ~azure.mgmt.appplatform.v2022_04_01.models.ClusterResourceProperties
        :keyword sku: Sku of the Service resource.
        :paramtype sku: ~azure.mgmt.appplatform.v2022_04_01.models.Sku
        """
        super(ServiceResource, self).__init__(location=location, tags=tags, **kwargs)
        self.properties = properties
        self.sku = sku


class ServiceResourceList(msrest.serialization.Model):
    """Object that includes an array of Service resources and a possible link for next set.

    :ivar value: Collection of Service resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ServiceResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[ServiceResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["ServiceResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of Service resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.ServiceResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(ServiceResourceList, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ServiceSpecification(msrest.serialization.Model):
    """Service specification payload.

    :ivar log_specifications: Specifications of the Log for Azure Monitoring.
    :vartype log_specifications: list[~azure.mgmt.appplatform.v2022_04_01.models.LogSpecification]
    :ivar metric_specifications: Specifications of the Metrics for Azure Monitoring.
    :vartype metric_specifications:
     list[~azure.mgmt.appplatform.v2022_04_01.models.MetricSpecification]
    """

    _attribute_map = {
        'log_specifications': {'key': 'logSpecifications', 'type': '[LogSpecification]'},
        'metric_specifications': {'key': 'metricSpecifications', 'type': '[MetricSpecification]'},
    }

    def __init__(
        self,
        *,
        log_specifications: Optional[List["LogSpecification"]] = None,
        metric_specifications: Optional[List["MetricSpecification"]] = None,
        **kwargs
    ):
        """
        :keyword log_specifications: Specifications of the Log for Azure Monitoring.
        :paramtype log_specifications:
         list[~azure.mgmt.appplatform.v2022_04_01.models.LogSpecification]
        :keyword metric_specifications: Specifications of the Metrics for Azure Monitoring.
        :paramtype metric_specifications:
         list[~azure.mgmt.appplatform.v2022_04_01.models.MetricSpecification]
        """
        super(ServiceSpecification, self).__init__(**kwargs)
        self.log_specifications = log_specifications
        self.metric_specifications = metric_specifications


class Sku(msrest.serialization.Model):
    """Sku of Azure Spring Apps.

    :ivar name: Name of the Sku.
    :vartype name: str
    :ivar tier: Tier of the Sku.
    :vartype tier: str
    :ivar capacity: Current capacity of the target resource.
    :vartype capacity: int
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'tier': {'key': 'tier', 'type': 'str'},
        'capacity': {'key': 'capacity', 'type': 'int'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = "S0",
        tier: Optional[str] = "Standard",
        capacity: Optional[int] = None,
        **kwargs
    ):
        """
        :keyword name: Name of the Sku.
        :paramtype name: str
        :keyword tier: Tier of the Sku.
        :paramtype tier: str
        :keyword capacity: Current capacity of the target resource.
        :paramtype capacity: int
        """
        super(Sku, self).__init__(**kwargs)
        self.name = name
        self.tier = tier
        self.capacity = capacity


class SkuCapacity(msrest.serialization.Model):
    """The SKU capacity.

    All required parameters must be populated in order to send to Azure.

    :ivar minimum: Required. Gets or sets the minimum.
    :vartype minimum: int
    :ivar maximum: Gets or sets the maximum.
    :vartype maximum: int
    :ivar default: Gets or sets the default.
    :vartype default: int
    :ivar scale_type: Gets or sets the type of the scale. Possible values include: "None",
     "Manual", "Automatic".
    :vartype scale_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.SkuScaleType
    """

    _validation = {
        'minimum': {'required': True},
    }

    _attribute_map = {
        'minimum': {'key': 'minimum', 'type': 'int'},
        'maximum': {'key': 'maximum', 'type': 'int'},
        'default': {'key': 'default', 'type': 'int'},
        'scale_type': {'key': 'scaleType', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        minimum: int,
        maximum: Optional[int] = None,
        default: Optional[int] = None,
        scale_type: Optional[Union[str, "SkuScaleType"]] = None,
        **kwargs
    ):
        """
        :keyword minimum: Required. Gets or sets the minimum.
        :paramtype minimum: int
        :keyword maximum: Gets or sets the maximum.
        :paramtype maximum: int
        :keyword default: Gets or sets the default.
        :paramtype default: int
        :keyword scale_type: Gets or sets the type of the scale. Possible values include: "None",
         "Manual", "Automatic".
        :paramtype scale_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.SkuScaleType
        """
        super(SkuCapacity, self).__init__(**kwargs)
        self.minimum = minimum
        self.maximum = maximum
        self.default = default
        self.scale_type = scale_type


class SourceUploadedUserSourceInfo(UploadedUserSourceInfo):
    """Uploaded Java source code binary for a deployment.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of the source uploaded.Constant filled by server.
    :vartype type: str
    :ivar version: Version of the source.
    :vartype version: str
    :ivar relative_path: Relative path of the storage which stores the source.
    :vartype relative_path: str
    :ivar artifact_selector: Selector for the artifact to be used for the deployment for
     multi-module projects. This should be
     the relative path to the target module/project.
    :vartype artifact_selector: str
    :ivar runtime_version: Runtime version of the source file.
    :vartype runtime_version: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'relative_path': {'key': 'relativePath', 'type': 'str'},
        'artifact_selector': {'key': 'artifactSelector', 'type': 'str'},
        'runtime_version': {'key': 'runtimeVersion', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        version: Optional[str] = None,
        relative_path: Optional[str] = None,
        artifact_selector: Optional[str] = None,
        runtime_version: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword version: Version of the source.
        :paramtype version: str
        :keyword relative_path: Relative path of the storage which stores the source.
        :paramtype relative_path: str
        :keyword artifact_selector: Selector for the artifact to be used for the deployment for
         multi-module projects. This should be
         the relative path to the target module/project.
        :paramtype artifact_selector: str
        :keyword runtime_version: Runtime version of the source file.
        :paramtype runtime_version: str
        """
        super(SourceUploadedUserSourceInfo, self).__init__(version=version, relative_path=relative_path, **kwargs)
        self.type = 'Source'  # type: str
        self.artifact_selector = artifact_selector
        self.runtime_version = runtime_version


class StackProperties(msrest.serialization.Model):
    """KPack ClusterStack properties payload.

    :ivar id: Id of the ClusterStack.
    :vartype id: str
    :ivar version: Version of the ClusterStack.
    :vartype version: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        version: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword id: Id of the ClusterStack.
        :paramtype id: str
        :keyword version: Version of the ClusterStack.
        :paramtype version: str
        """
        super(StackProperties, self).__init__(**kwargs)
        self.id = id
        self.version = version


class SupportedBuildpackResource(ProxyResource):
    """Supported buildpack resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Supported buildpack resource properties.
    :vartype properties:
     ~azure.mgmt.appplatform.v2022_04_01.models.SupportedBuildpackResourceProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'SupportedBuildpackResourceProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["SupportedBuildpackResourceProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Supported buildpack resource properties.
        :paramtype properties:
         ~azure.mgmt.appplatform.v2022_04_01.models.SupportedBuildpackResourceProperties
        """
        super(SupportedBuildpackResource, self).__init__(**kwargs)
        self.properties = properties


class SupportedBuildpackResourceProperties(msrest.serialization.Model):
    """Supported buildpack resource properties.

    :ivar buildpack_id: The id of supported buildpack.
    :vartype buildpack_id: str
    """

    _attribute_map = {
        'buildpack_id': {'key': 'buildpackId', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        buildpack_id: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword buildpack_id: The id of supported buildpack.
        :paramtype buildpack_id: str
        """
        super(SupportedBuildpackResourceProperties, self).__init__(**kwargs)
        self.buildpack_id = buildpack_id


class SupportedBuildpacksCollection(msrest.serialization.Model):
    """Object that includes an array of supported buildpacks resources and a possible link for next set.

    :ivar value: Collection of supported buildpacks resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.SupportedBuildpackResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[SupportedBuildpackResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["SupportedBuildpackResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of supported buildpacks resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.SupportedBuildpackResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(SupportedBuildpacksCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class SupportedRuntimeVersion(msrest.serialization.Model):
    """Supported deployment runtime version descriptor.

    :ivar value: The raw value which could be passed to deployment CRUD operations. Possible values
     include: "Java_8", "Java_11", "Java_17", "NetCore_31".
    :vartype value: str or ~azure.mgmt.appplatform.v2022_04_01.models.SupportedRuntimeValue
    :ivar platform: The platform of this runtime version (possible values: "Java" or ".NET").
     Possible values include: "Java", ".NET Core".
    :vartype platform: str or ~azure.mgmt.appplatform.v2022_04_01.models.SupportedRuntimePlatform
    :ivar version: The detailed version (major.minor) of the platform.
    :vartype version: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': 'str'},
        'platform': {'key': 'platform', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[Union[str, "SupportedRuntimeValue"]] = None,
        platform: Optional[Union[str, "SupportedRuntimePlatform"]] = None,
        version: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: The raw value which could be passed to deployment CRUD operations. Possible
         values include: "Java_8", "Java_11", "Java_17", "NetCore_31".
        :paramtype value: str or ~azure.mgmt.appplatform.v2022_04_01.models.SupportedRuntimeValue
        :keyword platform: The platform of this runtime version (possible values: "Java" or ".NET").
         Possible values include: "Java", ".NET Core".
        :paramtype platform: str or ~azure.mgmt.appplatform.v2022_04_01.models.SupportedRuntimePlatform
        :keyword version: The detailed version (major.minor) of the platform.
        :paramtype version: str
        """
        super(SupportedRuntimeVersion, self).__init__(**kwargs)
        self.value = value
        self.platform = platform
        self.version = version


class SupportedStackResource(ProxyResource):
    """Supported stack resource payload.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource Id for the resource.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :ivar system_data: Metadata pertaining to creation and last modification of the resource.
    :vartype system_data: ~azure.mgmt.appplatform.v2022_04_01.models.SystemData
    :ivar properties: Supported stack resource properties.
    :vartype properties:
     ~azure.mgmt.appplatform.v2022_04_01.models.SupportedStackResourceProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'system_data': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'system_data': {'key': 'systemData', 'type': 'SystemData'},
        'properties': {'key': 'properties', 'type': 'SupportedStackResourceProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["SupportedStackResourceProperties"] = None,
        **kwargs
    ):
        """
        :keyword properties: Supported stack resource properties.
        :paramtype properties:
         ~azure.mgmt.appplatform.v2022_04_01.models.SupportedStackResourceProperties
        """
        super(SupportedStackResource, self).__init__(**kwargs)
        self.properties = properties


class SupportedStackResourceProperties(msrest.serialization.Model):
    """Supported stack resource properties.

    :ivar stack_id: The id of supported stack.
    :vartype stack_id: str
    :ivar version: The version of supported stack.
    :vartype version: str
    """

    _attribute_map = {
        'stack_id': {'key': 'stackId', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        stack_id: Optional[str] = None,
        version: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword stack_id: The id of supported stack.
        :paramtype stack_id: str
        :keyword version: The version of supported stack.
        :paramtype version: str
        """
        super(SupportedStackResourceProperties, self).__init__(**kwargs)
        self.stack_id = stack_id
        self.version = version


class SupportedStacksCollection(msrest.serialization.Model):
    """Object that includes an array of supported stacks resources and a possible link for next set.

    :ivar value: Collection of supported stacks resources.
    :vartype value: list[~azure.mgmt.appplatform.v2022_04_01.models.SupportedStackResource]
    :ivar next_link: URL client should use to fetch the next page (per server side paging).
     It's null for now, added for future use.
    :vartype next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[SupportedStackResource]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["SupportedStackResource"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword value: Collection of supported stacks resources.
        :paramtype value: list[~azure.mgmt.appplatform.v2022_04_01.models.SupportedStackResource]
        :keyword next_link: URL client should use to fetch the next page (per server side paging).
         It's null for now, added for future use.
        :paramtype next_link: str
        """
        super(SupportedStacksCollection, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class SystemData(msrest.serialization.Model):
    """Metadata pertaining to creation and last modification of the resource.

    :ivar created_by: The identity that created the resource.
    :vartype created_by: str
    :ivar created_by_type: The type of identity that created the resource. Possible values include:
     "User", "Application", "ManagedIdentity", "Key".
    :vartype created_by_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.CreatedByType
    :ivar created_at: The timestamp of resource creation (UTC).
    :vartype created_at: ~datetime.datetime
    :ivar last_modified_by: The identity that last modified the resource.
    :vartype last_modified_by: str
    :ivar last_modified_by_type: The type of identity that last modified the resource. Possible
     values include: "User", "Application", "ManagedIdentity", "Key".
    :vartype last_modified_by_type: str or
     ~azure.mgmt.appplatform.v2022_04_01.models.LastModifiedByType
    :ivar last_modified_at: The timestamp of resource modification (UTC).
    :vartype last_modified_at: ~datetime.datetime
    """

    _attribute_map = {
        'created_by': {'key': 'createdBy', 'type': 'str'},
        'created_by_type': {'key': 'createdByType', 'type': 'str'},
        'created_at': {'key': 'createdAt', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'str'},
        'last_modified_by_type': {'key': 'lastModifiedByType', 'type': 'str'},
        'last_modified_at': {'key': 'lastModifiedAt', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        *,
        created_by: Optional[str] = None,
        created_by_type: Optional[Union[str, "CreatedByType"]] = None,
        created_at: Optional[datetime.datetime] = None,
        last_modified_by: Optional[str] = None,
        last_modified_by_type: Optional[Union[str, "LastModifiedByType"]] = None,
        last_modified_at: Optional[datetime.datetime] = None,
        **kwargs
    ):
        """
        :keyword created_by: The identity that created the resource.
        :paramtype created_by: str
        :keyword created_by_type: The type of identity that created the resource. Possible values
         include: "User", "Application", "ManagedIdentity", "Key".
        :paramtype created_by_type: str or ~azure.mgmt.appplatform.v2022_04_01.models.CreatedByType
        :keyword created_at: The timestamp of resource creation (UTC).
        :paramtype created_at: ~datetime.datetime
        :keyword last_modified_by: The identity that last modified the resource.
        :paramtype last_modified_by: str
        :keyword last_modified_by_type: The type of identity that last modified the resource. Possible
         values include: "User", "Application", "ManagedIdentity", "Key".
        :paramtype last_modified_by_type: str or
         ~azure.mgmt.appplatform.v2022_04_01.models.LastModifiedByType
        :keyword last_modified_at: The timestamp of resource modification (UTC).
        :paramtype last_modified_at: ~datetime.datetime
        """
        super(SystemData, self).__init__(**kwargs)
        self.created_by = created_by
        self.created_by_type = created_by_type
        self.created_at = created_at
        self.last_modified_by = last_modified_by
        self.last_modified_by_type = last_modified_by_type
        self.last_modified_at = last_modified_at


class TemporaryDisk(msrest.serialization.Model):
    """Temporary disk payload.

    :ivar size_in_gb: Size of the temporary disk in GB.
    :vartype size_in_gb: int
    :ivar mount_path: Mount path of the temporary disk.
    :vartype mount_path: str
    """

    _validation = {
        'size_in_gb': {'maximum': 5, 'minimum': 0},
    }

    _attribute_map = {
        'size_in_gb': {'key': 'sizeInGB', 'type': 'int'},
        'mount_path': {'key': 'mountPath', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        size_in_gb: Optional[int] = None,
        mount_path: Optional[str] = "/tmp",
        **kwargs
    ):
        """
        :keyword size_in_gb: Size of the temporary disk in GB.
        :paramtype size_in_gb: int
        :keyword mount_path: Mount path of the temporary disk.
        :paramtype mount_path: str
        """
        super(TemporaryDisk, self).__init__(**kwargs)
        self.size_in_gb = size_in_gb
        self.mount_path = mount_path


class TestKeys(msrest.serialization.Model):
    """Test keys payload.

    :ivar primary_key: Primary key.
    :vartype primary_key: str
    :ivar secondary_key: Secondary key.
    :vartype secondary_key: str
    :ivar primary_test_endpoint: Primary test endpoint.
    :vartype primary_test_endpoint: str
    :ivar secondary_test_endpoint: Secondary test endpoint.
    :vartype secondary_test_endpoint: str
    :ivar enabled: Indicates whether the test endpoint feature enabled or not.
    :vartype enabled: bool
    """

    _attribute_map = {
        'primary_key': {'key': 'primaryKey', 'type': 'str'},
        'secondary_key': {'key': 'secondaryKey', 'type': 'str'},
        'primary_test_endpoint': {'key': 'primaryTestEndpoint', 'type': 'str'},
        'secondary_test_endpoint': {'key': 'secondaryTestEndpoint', 'type': 'str'},
        'enabled': {'key': 'enabled', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        primary_key: Optional[str] = None,
        secondary_key: Optional[str] = None,
        primary_test_endpoint: Optional[str] = None,
        secondary_test_endpoint: Optional[str] = None,
        enabled: Optional[bool] = None,
        **kwargs
    ):
        """
        :keyword primary_key: Primary key.
        :paramtype primary_key: str
        :keyword secondary_key: Secondary key.
        :paramtype secondary_key: str
        :keyword primary_test_endpoint: Primary test endpoint.
        :paramtype primary_test_endpoint: str
        :keyword secondary_test_endpoint: Secondary test endpoint.
        :paramtype secondary_test_endpoint: str
        :keyword enabled: Indicates whether the test endpoint feature enabled or not.
        :paramtype enabled: bool
        """
        super(TestKeys, self).__init__(**kwargs)
        self.primary_key = primary_key
        self.secondary_key = secondary_key
        self.primary_test_endpoint = primary_test_endpoint
        self.secondary_test_endpoint = secondary_test_endpoint
        self.enabled = enabled


class TriggeredBuildResult(msrest.serialization.Model):
    """The build result triggered by a build.

    :ivar id: The unique build id of this build result.
    :vartype id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword id: The unique build id of this build result.
        :paramtype id: str
        """
        super(TriggeredBuildResult, self).__init__(**kwargs)
        self.id = id


class ValidationMessages(msrest.serialization.Model):
    """Validate messages of the configuration service git repositories.

    :ivar name: The name of the configuration service git repository.
    :vartype name: str
    :ivar messages: Detailed validation messages.
    :vartype messages: list[str]
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'messages': {'key': 'messages', 'type': '[str]'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        messages: Optional[List[str]] = None,
        **kwargs
    ):
        """
        :keyword name: The name of the configuration service git repository.
        :paramtype name: str
        :keyword messages: Detailed validation messages.
        :paramtype messages: list[str]
        """
        super(ValidationMessages, self).__init__(**kwargs)
        self.name = name
        self.messages = messages
